/*
 * wm8904.h  --  WM8904 Soc Audio driver
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */


/*----------------------------------------------------------------------------
 *         Definitions
 *----------------------------------------------------------------------------*/
#define WM8904_CSB_STATE            (0x0 << 0)

/** Slave address */
#define WM8904_SLAVE_ADDRESS                (0x34)


/** Reset register*/
#define WM8904_REG_RESET                           0x00

/** Bias control 0 register*/
#define WM8904_REG_BIAS_CTRL0                      0x04

/** VMID control 0 register*/
#define WM8904_REG_VMID_CTRL0                      0x05

/** MIC Bias control 0 register*/
#define WM8904_REG_MICBIAS_CTRL0                   0x06

/** Bias control 1 register*/
#define WM8904_REG_BIAS_CTRL1                      0x07

/** Power management control 0 register*/
#define WM8904_REG_POWER_MANG0                     0x0C
/** Power management control 2 register*/
#define WM8904_REG_POWER_MANG2                     0x0E
/** Power management control 3 register*/
#define WM8904_REG_POWER_MANG3                     0x0F
/** Power management control 6 register*/
#define WM8904_REG_POWER_MANG6                     0x12

/** Clock rate0 register*/
#define WM8904_REG_CLOCK_RATE0                     0x14
/** Clock rate1 register*/
#define WM8904_REG_CLOCK_RATE1                     0x15

/** Clock rate2 register*/
#define WM8904_REG_CLOCK_RATE2                     0x16

/** Audio interface0 register*/
#define WM8904_REG_AUD_INF0                        0x18

/** Audio interface1 register*/
#define WM8904_REG_AUD_INF1                        0x19
/** Audio interface2 register*/
#define WM8904_REG_AUD_INF2                        0x1A
/** Audio interface3 register*/
#define WM8904_REG_AUD_INF3                        0x1B

/** DAC Digital Volume Left register*/
#define WM8904_REG_AUD_DAC_LVOL                        0x1E
/** DAC Digital Volume Right register*/
#define WM8904_REG_AUD_DAC_RVOL                        0x1F

/** ADC digital 0 register*/
#define WM8904_REG_ADC_DIG0                        0x20
/** ADC digital 1 register*/
#define WM8904_REG_ADC_DIG1                        0x21

/** Analogue left input 0 register*/
#define WM8904_REG_ANALOGUE_LIN0                   0x2C
/** Analogue right input 0 register*/
#define WM8904_REG_ANALOGUE_RIN0                   0x2D

/** Analogue left input 1 register*/
#define WM8904_REG_ANALOGUE_LIN1                   0x2E
/** Analogue right input 1 register*/
#define WM8904_REG_ANALOGUE_RIN1                   0x2F

/** Analogue left output 1 register*/
#define WM8904_REG_ANALOGUE_LOUT1                  0x39
/** Analogue right output 1 register*/
#define WM8904_REG_ANALOGUE_ROUT1                  0x3A

/** Analogue left output 2 register*/
#define WM8904_REG_ANALOGUE_LOUT2                  0x3B
/** Analogue right output 2 register*/
#define WM8904_REG_ANALOGUE_ROUT2                  0x3C

/** Analogue output 12 ZC register*/
#define WM8904_REG_ANALOGUE_OUT12ZC                0x3D

/** DC servo 0 register*/
#define WM8904_REG_DC_SERVO0                       0x43

/** Analogue HP 0 register*/
#define WM8904_REG_ANALOGUE_HP0                    0x5A

/** Charge pump 0 register*/
#define WM8904_REG_CHARGE_PUMP0                    0x62

/** Class W 0 register*/
#define WM8904_REG_CLASS0                          0x68

/** FLL control 1 register*/
#define WM8904_REG_FLL_CRTL1                       0x74
/** FLL control 2 register*/
#define WM8904_REG_FLL_CRTL2                       0x75
/** FLL control 3 register*/
#define WM8904_REG_FLL_CRTL3                       0x76
/** FLL control 4 register*/
#define WM8904_REG_FLL_CRTL4                       0x77
/** FLL control 5 register*/
#define WM8904_REG_FLL_CRTL5                       0x78

/** DUMMY register*/
#define WM8904_REG_END                             0xFF

struct WM8904_REG
{
	unsigned char addr;
	unsigned short data;
};

#if 0
struct WM8904_REG WM8904_InitValue[]= 
{
    {0x00, 0x0000},
    {0x6c, 0x0100},
    {0x6f, 0x0100},
    {0x14, 0x845E},
    {0x39, 0x0039},
    {0x3A, 0x00B9},
    {0x21, 0x0000},
    {0x68, 0x0005},
};
#else
struct WM8904_REG WM8904_InitValue[]= 
{
    {WM8904_REG_RESET          ,   0x0000}, /* Software reset */
    {WM8904_REG_BIAS_CTRL0     ,   0x0019}, /* POBCTRL=1, ISEL=10, STARTUP_BIAS_ENA=0, BIAS_ENA=1  */
    {WM8904_REG_VMID_CTRL0     ,   0x0043}, /* VMID_BUF_ENA=1, VMID_RES=01, VMID_ENA=1   */
    {WM8904_REG_MICBIAS_CTRL0  ,   0x0003}, /* MICDET_ENA=1, MICBIAS_ENA=1   */
    {WM8904_REG_BIAS_CTRL1     ,   0xC000}, /* ?  */
    {WM8904_REG_POWER_MANG0    ,   0x0003}, /* INL_ENA=1, INR_ENA=1  */
    {WM8904_REG_POWER_MANG2    ,   0x0003}, /* HPL_PGA_ENA=1, HPR_PGA_ENA=1  */
    {WM8904_REG_POWER_MANG6    ,   0x000F}, /* DACL_ENA=1, DACR_ENA=1, ADCL_ENA=1, ADCR_ENA=1  */
    {WM8904_REG_CLOCK_RATE0    ,   0x845E}, /* TOCLK_RATE_DIV16=0,TOCLK_RATE_X4=0, SR_MODE=0, MCLK_DIV=0  */
    {WM8904_REG_CLOCK_RATE2    ,   0x4006}, /* SYSCLK_SRC=1, CLK_SYS_ENA=1, CLK_DSP_ENA=1  */
    {WM8904_REG_AUD_INF1       ,   0x400A},//0x4002},//0x404A}, /* AIFADC_TCM=0, AIFADC_TCM_CHAN=0, BCLK_DIR=1 */
    {WM8904_REG_AUD_INF3       ,   0x0040},//0x0840}, /* LRCLK=1, LRCLK_RATE=0x40 */
    {WM8904_REG_ADC_DIG1       ,   0x0000}, /* DAC_MCNO=0, DAC_SB_FILT=0, DAC_MUTERATE=0, DAC_MUTE=0 */
    {WM8904_REG_ANALOGUE_LIN0  ,   0x0005}, /* LINMUTE=0, LIN_VOL= 0 db */
    {WM8904_REG_ANALOGUE_RIN0  ,   0x0005}, /* RINMUTE=0, RIN_VOL= 0 db */
    {WM8904_REG_ANALOGUE_LIN1  ,   0x0010}, /* IN2L */
    {WM8904_REG_ANALOGUE_RIN1  ,   0x0010}, /* IN2R*/
    {WM8904_REG_ANALOGUE_ROUT1 ,   0x00AD}, /* HPOUTR_MUTE=0, HPOUTRZC=0, HPOUTR_VOL=0x1D */
    {WM8904_REG_DC_SERVO0      ,   0x0003}, /* DCS_ENA_CHAN_1=1, DCS_ENA_CHAN_0=1 */
    {WM8904_REG_ANALOGUE_HP0   ,   0x00FF}, /* HPL_RMV_SHORT=1, HPL_ENA_OUTp=1, HPL_ENA_DLY=1, HPL_ENA=1 */
    {WM8904_REG_CHARGE_PUMP0   ,   0x0001}, /* CP_ENA=1 */
    {WM8904_REG_CLASS0         ,   0x0005}, /* CP_DYN_PWR=1 */
    {WM8904_REG_FLL_CRTL1      ,   0x0000}, /* FLL_FRACN_ENA=0, FLL_ENA=0 */
    {WM8904_REG_FLL_CRTL1      ,   0x0005}, /* FLL_FRACN_ENA=1, FLL_ENA=1 */
    {WM8904_REG_FLL_CRTL2      ,   0x0704}, /* FLL_FRATIO=4, FLL_OUTDIV= 7 */
    {WM8904_REG_FLL_CRTL3      ,   0x8000}, /* Fractional multiply for Fref = 0x8000 */
    {WM8904_REG_FLL_CRTL4      ,   0x1760}, /* FLL_GAIN=0, FLL_N=0x176 */
    {WM8904_REG_END            ,   0x55AA}
};
#endif

#define WM8904_INIT_SIZE (sizeof(WM8904_InitValue)/sizeof(WM8904_InitValue[0]))

struct WM8904_REG WM8904_Set[]= 
{
    { WM8904_REG_ANALOGUE_LIN0 ,   0x0012},
    { WM8904_REG_ANALOGUE_RIN0 ,   0x0012},
    { WM8904_REG_ANALOGUE_LIN1 ,   0x0000},
    { WM8904_REG_ANALOGUE_RIN1 ,   0x0000},
};
