/*
 * wm8960.h  --  WM8960 Soc Audio driver
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
/*----------------------------------------------------------------------------
 *         Definitions
 *----------------------------------------------------------------------------*/
//#define WM8960_CSB_STATE            (0x0 << 0)
//#define WM8960_SLAVE_ADDRESS        (0x1a | WM8731_CSB_STATE)

/** Slave address */
#define WM8960_SLAVE_ADDRESS        (0x34>>1)

/* WM8960 register space */


#define WM8960_CACHEREGNUM      56

#define WM8960_LINVOL           0x0
#define WM8960_RINVOL           0x1
#define WM8960_LOUT1            0x2
#define WM8960_ROUT1            0x3
#define WM8960_CLOCK1           0x4
#define WM8960_DACCTL1          0x5
#define WM8960_DACCTL2          0x6
#define WM8960_IFACE1           0x7
#define WM8960_CLOCK2           0x8
#define WM8960_IFACE2           0x9
#define WM8960_LDAC             0xa
#define WM8960_RDAC             0xb

#define WM8960_RESET            0xf
#define WM8960_3D               0x10
#define WM8960_ALC1             0x11
#define WM8960_ALC2             0x12
#define WM8960_ALC3             0x13
#define WM8960_NOISEG           0x14
#define WM8960_LADC             0x15
#define WM8960_RADC             0x16
#define WM8960_ADDCTL1          0x17
#define WM8960_ADDCTL2          0x18
#define WM8960_POWER1           0x19
#define WM8960_POWER2           0x1a
#define WM8960_ADDCTL3          0x1b
#define WM8960_APOP1            0x1c
#define WM8960_APOP2            0x1d

#define WM8960_LINPATH          0x20
#define WM8960_RINPATH          0x21
#define WM8960_LOUTMIX1         0x22
                            
#define WM8960_ROUTMIX2         0x25
#define WM8960_MONOMIX1         0x26
#define WM8960_MONOMIX2         0x27
#define WM8960_LOUT2            0x28
#define WM8960_ROUT2            0x29
#define WM8960_MONO             0x2a
#define WM8960_INBMIX1          0x2b
#define WM8960_INBMIX2          0x2c
#define WM8960_BYPASS1          0x2d
#define WM8960_BYPASS2          0x2e
#define WM8960_POWER3           0x2f
#define WM8960_ADDCTL4          0x30
#define WM8960_CLASSD1          0x31

#define WM8960_CLASSD3          0x33
#define WM8960_PLLN             0x34
#define WM8960_PLLK1            0x35
#define WM8960_PLLK2            0x36
#define WM8960_PLLK3            0x37

#define WM8960_DELAY            0xFF

/*
 * WM8960 Clock dividers
 */
#define WM8960_SYSCLKDIV        0
#define WM8960_DACDIV           1
#define WM8960_OPCLKDIV         2
#define WM8960_DCLKDIV          3
#define WM8960_TOCLKSEL         4
#define WM8960_SYSCLKSEL        5

#define WM8960_SYSCLK_DIV_1     (0 << 1)
#define WM8960_SYSCLK_DIV_2     (2 << 1)

#define WM8960_SYSCLK_MCLK      (0 << 0)
#define WM8960_SYSCLK_PLL       (1 << 0)

#define WM8960_DAC_DIV_1        (0 << 3)
#define WM8960_DAC_DIV_1_5      (1 << 3)
#define WM8960_DAC_DIV_2        (2 << 3)
#define WM8960_DAC_DIV_3        (3 << 3)
#define WM8960_DAC_DIV_4        (4 << 3)
#define WM8960_DAC_DIV_5_5      (5 << 3)
#define WM8960_DAC_DIV_6        (6 << 3)

#define WM8960_DCLK_DIV_1_5     (0 << 6)
#define WM8960_DCLK_DIV_2       (1 << 6)
#define WM8960_DCLK_DIV_3       (2 << 6)
#define WM8960_DCLK_DIV_4       (3 << 6)
#define WM8960_DCLK_DIV_6       (4 << 6)
#define WM8960_DCLK_DIV_8       (5 << 6)
#define WM8960_DCLK_DIV_12      (6 << 6)
#define WM8960_DCLK_DIV_16      (7 << 6)

#define WM8960_TOCLK_F19        (0 << 1)
#define WM8960_TOCLK_F21        (1 << 1)

#define WM8960_OPCLK_DIV_1      (0 << 0)
#define WM8960_OPCLK_DIV_2      (1 << 0)
#define WM8960_OPCLK_DIV_3      (2 << 0)
#define WM8960_OPCLK_DIV_4      (3 << 0)
#define WM8960_OPCLK_DIV_5_5    (4 << 0)
#define WM8960_OPCLK_DIV_6      (5 << 0)

struct WM8960_REG
{
	unsigned char addr;
	unsigned short data;
};

struct WM8960_REG WM8960_InitValue[]= 
{
#if 1
    {((WM8960_RESET     )),   0x000},        //RESET R15
    {WM8960_DELAY, 			    0x64},        //Delay (50mSeconds) to allow HP amps to settle
    {((WM8960_POWER1      )),   0x0c2},       //VMID= 50K,Enable VREF
    //{((WM8960_POWER2      )),   0x198},
	{WM8960_DELAY, 			0x64},                //Delay (50mSeconds) to allow HP amps to settle
	{((WM8960_POWER2      )),   0x1fa},
    {((WM8960_POWER3      )),   0x00c},
    {((WM8960_CLASSD1     )),   0x077},//0x077},//0x0f7},
	{((WM8960_CLASSD3     )),   0x09a},
    //{((WM8960_CLASSD3     )),   0x0ad},     // maximum
    //{((WM8960_CLASSD3     )),   0x09a},     // maximum
    {((WM8960_LOUTMIX1    )),   0x100},
    {((WM8960_ROUTMIX2     )),   0x100},
    {((WM8960_LOUT2      )),   0x179},
    {((WM8960_ROUT2      )),   0x179},
    {((WM8960_IFACE1      )),   0x002},
    {((WM8960_DACCTL1       )),   0x004},
	{((WM8960_MONO     )), 0x000},        //MONO OUT VOLUME
	{((WM8960_MONOMIX2    )), 0x080},        //MONO OUT MIXER enable
	{((WM8960_LOUTMIX1     )), 0x050},        //LD2LO, LI2LO Disable
	{((WM8960_BYPASS1     )), 0x00}, //LB2LO Disable //0x90},        //LB2LO Enable Path
	{((WM8960_ROUTMIX2     )), 0x110},        //RD2RO Enable Path, RI2RO Disable Path
	{((WM8960_BYPASS2     )), 0x90},        //RB2RO Enable Path
	
	{((WM8960_POWER1     )), 0xf2},        //AINL, AINR Enable
	{((WM8960_POWER3      )), 0x3C},        //LMIC, RMIC Enable
	//{((WM8960_LINVOL     )), 0xbf},        //LINVOL set MAX
	//{((WM8960_LINPATH     )), 0x138},        //LMN1, LMP2, LMIC2B
	//{((WM8960_INBMIX1     )), 0x7e},        //LIN2BOOST MAX
	//{((WM8960_ALC1     )), 0x1fb},        //RPGA, LPGA Enable
	{((WM8960_LINVOL     )), 0x15f},        //LINVOL set MAX, LINMUT E //15f(6db) -> 167(12db) -> 17f(30db)
	{((WM8960_RINVOL     )), 0x17f},        //RINVOL set MAX, LINMUT E 
	{((WM8960_LINPATH     )), 0x158},        //LMN1, LMP2, LMIC2B
	{((WM8960_RINPATH     )), 0x14b},        //RMN1, RMP2, RMIC2B	//15b(rmic 13db) -> 17b(rmic 29db)
	{((WM8960_INBMIX1     )), 0x00},        //LIN2BOOST MUTE
	{((WM8960_INBMIX2     )), 0x00},        //RIN2BOOST MUTE
	
#else
    {((WM8960_APOP1      )), 0x94},        //Enable POBCTRL, SOFT_ST and BUFDCOPEN
    {((WM8960_APOP2      )), 0x40},        //Enable DISOP
    {WM8960_DELAY, 			0x19},                        //Delay (400mS) to remove any residual charge on HP output.    
    {((WM8960_POWER2     )), 0x7f},        //Enable LOUT1 and ROUT1
    {((WM8960_APOP2      )), 0x00},        //Enable DISOP
    {((WM8960_POWER1     )), 0xc0},        //Enable VMID SEL = 2x50K Ohm Dividere
    {WM8960_DELAY, 			0x64},                //Delay (50mSeconds) to allow HP amps to settle
    {((WM8960_APOP1      )), 0x01},        //Enable POBCTRL, SOFT_ST and BUFDCOPEN

    {((WM8960_POWER2     )), 0x1f8},        //Enable LOUT1 and ROUT1
    {((WM8960_ADDCTL1    )), 0x1C1},        //DMONOMIX = 0, Stereo
    {((WM8960_POWER3      )), 0x0C},        //Enable left and right output mixers
    {((WM8960_LOUTMIX1    )), 0x150},        //Enable Left DAC to left mixer (LINPUT3 to Output Mixer)
    {((WM8960_ROUTMIX2    )), 0x150},        //Enable Right DAC to right mixer    
    {((WM8960_ADDCTL4    )), 0x00},        // ADCLRC used for Jack Detect Input
    {((WM8960_ADDCTL2    )), 0x40},        // HPDETECT HIGH = Speaker 
    {((WM8960_ADDCTL3    )), 0x08},        // OUT3CAP 

//PLL 12.288MHz     SysCLK 12.288MHz    F2 98.304 MHz
    //CLKSEL = 0
    //MS = 0
    //WL = 10
    //SYSCLKDIV =0
    //ADCDIV=000
    //DACDIV=000
    {((WM8960_CLOCK1      )), 0x00}, // DACDIV, CLKSEL
    {((WM8960_IFACE1      )), 0x00},

    {((WM8960_LOUT1       )), 0x17f},        //LOUT1VOL (HP) = -20dB
    {((WM8960_ROUT1       )), 0x17f},        //ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels
    {((WM8960_LOUT2       )), 0x17f},        //LOUT2VOL (HP) = -20dB
    {((WM8960_ROUT2       )), 0x17f},        //ROUT2VOL (HP) = -20dB, Enable OUT2VU, load volume settings to both left and right channels
    {((WM8960_CLASSD3     )), 0x09},        //Left and Right Speakers Enabled    
    {((WM8960_IFACE1      )), 0x02},        //
//    {((WM8960_CLASSD1     )), 0xb7},        //Left and Right Speakers Enabled    
//	{((WM8960_CLASSD1 	)), 0x77},		  //Left and Right Speakers Enabled    // jung hwan 100726 ( SPK Left Only
	{((WM8960_CLASSD1 	)), 0xf7},		  //Left and Right Speakers Enabled

    {((WM8960_DACCTL1     )), 0x04},        //DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)
	  
#if 1
	{((WM8960_LOUTMIX1     )), 0x050},        //LD2LO, LI2LO Disable
	{((WM8960_BYPASS1     )), 0x90},        //LB2LO Enable Path
	{((WM8960_ROUTMIX2     )), 0x190},        //RD2RO Enable Path, RI2RO Enable Path
	{((WM8960_BYPASS2     )), 0x90},        //RB2RO Enable Path
	
	{((WM8960_POWER1     )), 0xf0},        //AINL, AINR Enable
	{((WM8960_POWER3      )), 0x2C},        //LMIC Enalbe
	{((WM8960_INBMIX2     )), 0x0e},        //RIN2BOOST MAX
	{((WM8960_LINVOL     )), 0x3f},        //LINVOL set MAX
	{((WM8960_LINPATH     )), 0x178},        //LMN1, LMP2, LMIC2B
	{((WM8960_INBMIX1     )), 0x0e},        //LIN2BOOST MAX
	//{((WM8960_ALC1     )), 0x1fb},        //RPGA, LPGA Enable
	
#endif
#endif

	
};
	/*
{

    {((WM8960_APOP1      <<1)+0x0), 0x94},        //Enable POBCTRL, SOFT_ST and BUFDCOPEN
    {((WM8960_APOP2      <<1)+0x0), 0x40},        //Enable DISOP
    {WM8960_DELAY, 					0x19},                        //Delay (400mS) to remove any residual charge on HP output.    
    {((WM8960_POWER2     <<1)+0x0), 0x7f},        //Enable LOUT1 and ROUT1
    {((WM8960_APOP2      <<1)+0x0), 0x00},        //Enable DISOP
    {((WM8960_POWER1     <<1)+0x0), 0xc0},        //Enable VMID SEL = 2x50K Ohm Dividere
    {WM8960_DELAY, 0x64},                //Delay (50mSeconds) to allow HP amps to settle
    {((WM8960_APOP1      <<1)+0x0), 0x01},        //Enable POBCTRL, SOFT_ST and BUFDCOPEN
    {((WM8960_POWER2     <<1)+0x1), 0xf8},        //Enable LOUT1 and ROUT1
    {((WM8960_ADDCTL1    <<1)+0x1), 0xC1},        //DMONOMIX = 0, Stereo
    {((WM8960_POWER3      <<1)+0x0), 0x0C},        //Enable left and right output mixers
    {((WM8960_LOUTMIX1    <<1)+0x1), 0x50},        //Enable Left DAC to left mixer (LINPUT3 to Output Mixer)
    {((WM8960_ROUTMIX2    <<1)+0x1), 0x50},        //Enable Right DAC to right mixer    
    {((WM8960_ADDCTL4    <<1)+0x0), 0x02},        // GPIO1 used for Jack Detect Input
    {((WM8960_ADDCTL2    <<1)+0x0), 0x60},        // HPDETECT HIGH = Speaker 
    {((WM8960_ADDCTL3    <<1)+0x0), 0x08},        // OUT3CAP 

//PLL 12.288MHz     SysCLK 12.288MHz    F2 98.304 MHz
    //CLKSEL = 0
    //MS = 0
    //WL = 10
    //SYSCLKDIV =0
    //ADCDIV=000
    //DACDIV=000
    {((WM8960_CLOCK1      <<1)+0x0), 0x00}, // DACDIV, CLKSEL
    {((WM8960_IFACE1      <<1)+0x0), 0x00},

    {((WM8960_LOUT1       <<1)+0x1), 0x7f},        //LOUT1VOL (HP) = -20dB
    {((WM8960_ROUT1       <<1)+0x1), 0x7f},        //ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels
    {((WM8960_LOUT2       <<1)+0x1), 0x7f},        //LOUT2VOL (HP) = -20dB
    {((WM8960_ROUT2       <<1)+0x1), 0x7f},        //ROUT2VOL (HP) = -20dB, Enable OUT2VU, load volume settings to both left and right channels
    {((WM8960_CLASSD3     <<1)+0x0), 0xcc},        //Left and Right Speakers Enabled    
    {((WM8960_IFACE1      <<1)+0x0), 0x02},        //
    {((WM8960_CLASSD1     <<1)+0x0), 0xf7},        //Left and Right Speakers Enabled    
    {((WM8960_DACCTL1     <<1)+0x0), 0x04}        //DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)
    
};
*/
#define WM8960_INIT_SIZE (sizeof(WM8960_InitValue)/sizeof(WM8960_InitValue[0]))

struct WM8960_REG WM8960_Power_Up[]= {

    /* WM8960_HP_DAC_Power_Up_Sequence_GBR_400R */
   /* DAC to HP Full Power Up Sequence, from COLD start */
//   {((29<<1)+0x0), 0x40},        //Enable DISOP
//   {0xff,0x19},                    //Delay (400mS) to remove any residual charge on HP output.    
   {((28)), 0x94},        //Enable POBCTRL, SOFT_ST and BUFDCOPEN
   {((26)), 0x60},        //Enable LOUT1 and ROUT1
   {0xff,0x32},                    //Delay (50mSeconds) to allow HP amps to settle

   {((28)), 0x00},        //Disable DLOUT1 DROUT1 and HPSTBY. BUFDCOPEN, POBCTRL, SOFT_ST remain enabled  
   {((25)), 0x80},        //Enable VMID SEL = 2x50K Ohm Dividere
   {WM8960_DELAY,0x64},                    //Delay (100mSeconds) to allow VMID to initially charge
   {((25)), 0xC0},        //Enable VREF. VMID SEL = 2x50K Ohm Divider remain.
   {((28)), 0x10},        //Disable POBCTRL and SOFT_ST. BUFDCOPEN remain enabled  (Delay from R25 = 080 to Disable POBCTRL and SOFT_ST >100mS)
   {((26)), 0x1E0},        //Enable DACL, DACR. LOUT1 and ROUT1 remain enabled 
   {((47)), 0x0C},        //Enable left and right output mixers
   {((34)), 0x150},        //Enable Left DAC to left mixer  18dB
   {((37)), 0x150},        //Enable Right DAC to right mixer
   {((02)), 0x7f},        //LOUT1VOL (HP) = -20dB
   {((03)), 0x17f},        //ROUT1VOL (HP) = -20dB, Enable OUT1VU, load volume settings to both left and right channels

   {((05)), 0x00}        //DAC Digital Soft Mute = Unmute (Delay from R25 = 080 to unmute >250mS)

};
#define WM8960_POWERUP_SIZE (sizeof(WM8960_Power_Up)/sizeof(WM8960_Power_Up[0]))

unsigned char WM8960_Power_Up_addr[]= {
    29,28,26,25,47,34,02,03,05
};
struct WM8960_REG WM8960_Power_Down[]= {
   {((05)), 0x08},        //DAC Digital Soft Mute = Mute
   {((28)), 0x94},        //Enable POBCTRL and BUFDCOPEN and SOFT_ST
   {((25)), 0x00},        //Disable VMID and VREF, (allow output to discharge with S-Curve)
   {WM8960_DELAY,0x19},                                           //Delay 400mS to discharge HP outputs.
   {((26)), 0x00},        //Disable DACL, DACR, LOUT and ROUT1
   {((15)), 0x00}        //Reset Device (default registers)            
};
#define WM8960_POWERDOWN_SIZE (sizeof(WM8960_Power_Down)/sizeof(WM8960_Power_Down[0]))

