#include <types.h>
#include <memory.h>
#include <device.h>
#include <interrupt.h>
#include <time.h>
#include <thread.h>
#include <kernel.h>
#include <lib/string.h>
#include <cache.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3XBD.h>

#include <mach/hal_interrupt.h>
#include <dev/wavedrv.h>
//#include <dev/waveprivdrv.h>
#include <dev/drvlib.h>
#include <mach/gpio.h>
#include "wave.h"
#include "WM8960.h"

#include <hal_panic.h>

#define DBG_LVL_WAV         0
#define DGB_LVL_IF          0
#define DGB_LVL_TWI         0

/** Frequency of the board main oscillator */
//#define BOARD_MAINOSC           12000000
/** Master clock frequency (when using board_lowlevel.c) */
//#define BOARD_MCK                ((unsigned long)((BOARD_MAINOSC / 3 / 2 / 3) * 200 ))
/** Master clock frequency in Hz */
#define SSC_MCK                 BOARD_MCK
/** Wav feature. */
#define SAMPLE_RATE             (44000)
#define SLOT_BY_FRAME           (2)
#define BITS_BY_SLOT            (16)
/** TWI clock */
#define TWI_CLOCK               10000


/* Send data on Frame changes */
#define I2S_SLAVE_TX_SETTING(nb_bit_by_slot, nb_slot_by_frame)(\
                             (SSC_TCMR_CKS_TK) |\
                             (SSC_TCMR_CKO_NONE) |\
                             (SSC_TCMR_START_RF_EDGE) |\
                             (SSC_TCMR_STTDLY(1)) |\
                             (SSC_TCMR_PERIOD(0)))
/* Send 1 data each time */
#define I2S_SLAVE_TX_FRM_SETTING(nb_bit_by_slot, nb_slot_by_frame)( \
                                 (SSC_TFMR_DATLEN(nb_bit_by_slot - 1)) |\
                                 (SSC_TFMR_MSBF) |\
                                 (SSC_TFMR_DATNB(0)) |\
                                 (SSC_TFMR_FSOS_NONE))
/* Read data on Frame changes */
#define I2S_SLAVE_RX_SETTING(nb_bit_by_slot, nb_slot_by_frame)(\
                             (SSC_RCMR_CKS_RK) |\
                             (SSC_RCMR_CKO_NONE) |\
                             (SSC_RCMR_START_RF_FALLING) |\
                             (SSC_RCMR_STTDLY(1)) |\
                             (SSC_RCMR_PERIOD(0)))
/* Read 1 data each time */
#define I2S_SLAVE_RX_FRM_SETTING(nb_bit_by_slot, nb_slot_by_frame)( \
                                 (SSC_RFMR_DATLEN(nb_bit_by_slot - 1)) |\
                                 (SSC_RFMR_MSBF) |\
                                 (SSC_RFMR_DATNB(0)) |\
                                 (SSC_RFMR_FSOS_NONE))

#if 1
#define I2S_MASTER_TX_SETTING(nb_bit_by_slot, nb_slot_by_frame)(\
							  (SSC_TCMR_CKS_MCK) |\
                              (SSC_TCMR_CKO_CONTINUOUS) |\
                              (SSC_TCMR_START_RF_FALLING) |\
                              (SSC_TCMR_STTDLY(1)) |\
                              (SSC_TCMR_PERIOD((nb_bit_by_slot * nb_slot_by_frame) / 2 - 1)))

#define I2S_MASTER_TX_FRM_SETTING(nb_bit_by_slot, nb_slot_by_frame)( \
                                  (SSC_TFMR_DATLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_TFMR_MSBF) |\
                                  (SSC_TFMR_DATNB(nb_slot_by_frame - 1)) |\
                                  (SSC_TFMR_FSLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_TFMR_FSOS_NEGATIVE))

#else
#define I2S_MASTER_TX_SETTING(nb_bit_by_slot, nb_slot_by_frame)(\
                              (SSC_TCMR_CKS_MCK) |\
                              (SSC_TCMR_CKO_CONTINUOUS) |\
                              (SSC_TCMR_START_RF_FALLING) |\
                              (SSC_TCMR_STTDLY(1)) |\
                              (SSC_TCMR_PERIOD((nb_bit_by_slot * nb_slot_by_frame) / 2 - 1)))

#define I2S_MASTER_TX_FRM_SETTING(nb_bit_by_slot, nb_slot_by_frame)( \
                                  (SSC_TFMR_DATLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_TFMR_MSBF) |\
                                  (SSC_TFMR_DATNB(nb_slot_by_frame - 1)) |\
                                  (SSC_TFMR_FSLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_TFMR_FSOS_NEGATIVE))
#endif
#define I2S_MASTER_RX_SETTING(nb_bit_by_slot, nb_slot_by_frame)(\
                              (SSC_RCMR_CKS_MCK) |\
                              (SSC_RCMR_CKO_CONTINUOUS) |\
                              (SSC_RCMR_START_RF_FALLING) |\
                              (SSC_RCMR_STTDLY(1)) |\
                              (SSC_RCMR_PERIOD((nb_bit_by_slot * nb_slot_by_frame) / 2 - 1)))

#define I2S_MASTER_RX_FRM_SETTING(nb_bit_by_slot, nb_slot_by_frame)( \
                                  (SSC_RFMR_DATLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_RFMR_MSBF) |\
                                  (SSC_RFMR_DATNB(nb_slot_by_frame - 1)) |\
                                  (SSC_RFMR_FSLEN(nb_bit_by_slot - 1)) |\
                                  (SSC_RFMR_FSOS_NEGATIVE))


#define TWITIMEOUTMAX 50000
								  

typedef enum
{
    DMA_CH_OUT      = 0x1,
    DMA_CH_IN       = 0x2
} DMA_CH_SELECT;


// for sound buffer
#define AUDIO_DMA_PAGE_SIZE        (4096)                    // Size in bytes
STATIC PBYTE            InputDMAPage[INPUT_DMA_PAGE_MAX];
STATIC PBYTE            OutputDMAPage[OUTPUT_DMA_PAGE_MAX];

#define AUDIO_DMA_BUFFER_SIZE        (AUDIO_DMA_PAGE_SIZE*(OUTPUT_DMA_BUFFER_MAX+INPUT_DMA_BUFFER_MAX))
#define DMA_ADDR_OFFSET         0x20000000

//----- Used to track DMA controllers status -----
#define DMA_CLEAR           0x00000000
#define DMA_DONEA           0x00000008
#define DMA_STRTA           0x00000010
#define DMA_DONEB           0x00000020
#define DMA_STRTB           0x00000040
#define DMA_BIU             0x00000080    // Determines which buffer is in use: (A=0, B=1)

static UINT32			nDmaChannelRx;
static UINT32			nDmaChannelTx;

PUBLIC DEVICECONFIGWAVE         WAV_DeviceConfig ={"WAV", 0, -1, NULL, NULL };
STATIC UINT32                   snSysIntr = 0;
//==============================================================================
// hardware dependent definition
//==============================================================================

STATIC UINT32           v_nVolume=0xeeeeeeee; //default to volume

STATIC BOOL             m_InputDMARunning = FALSE;
STATIC UINT32           m_InputDMAStatus;

STATIC BOOL             m_OutputDMARunning = FALSE;
STATIC UINT32           m_OutputDMAStatus;

STATIC BOOL             m_bOutputMute = FALSE;
STATIC BOOL             m_bInputMute = FALSE;

#define DMA_CH_MIC 2
#define DMA_CH_OUT 1


//==============================================================================
#define I2C_CODEC_ID	        (0x34>>1)
#define I2C_CODEC_SPEED         100

#define WM8580_MAX_REGISTER_COUNT                    56
STATIC USHORT   m_WM8580_SFR_Table[WM8580_MAX_REGISTER_COUNT];


//==============================================================================


STATIC BOOL CODEC_PowerControl( VOID );
STATIC BOOL CODEC_MuteControl(  DWORD channel, BOOL bMute );
STATIC VOID CODEC_PowerOn( VOID );
STATIC VOID CODEC_PowerOff( VOID );
STATIC VOID CODEC_WriteRegister(UCHAR Reg, USHORT Val);
STATIC USHORT CODEC_ReadRegister(UCHAR Reg);

//==============================================================================
// from here, add the hardware specific routines
//==============================================================================
//==============================================================================

STATIC AT91PS_PIO pPIOD_Base = (AT91PS_PIO)AT91C_BASE_PIOA;
#define I2C_PIO_BASE				(AT91C_BASE_PIOA)
#define BIT_I2CSCL					(AT91C_PIO_PA31)
#define BIT_I2CSDA					(AT91C_PIO_PA30)


STATIC VOID DELAY( INT32 Cnt )
{
    volatile INT32   i;
    for( i = 0 ; i < Cnt;  )
	{
		i++;
	}
}

STATIC VOID
Start_I2C( VOID )
{
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSCL );
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSDA );

    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSCL );
    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSDA );

	DELAY(10);
	
    AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSDA );
	DELAY(30);
    AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSCL );
	DELAY(20);
}

STATIC VOID
Stop_I2C( VOID )
{
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSCL );
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSDA );

										
    AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSDA );
	DELAY( 20 );
    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSCL );
	DELAY(30);
    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSDA );
	DELAY(20);
	
}


STATIC BOOL
Write_I2C_Data( BYTE Data )
{
	WORD wClockCnt=0, wBitPointer=0;
	LONG Count=0;
	wBitPointer=0x80;

    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSCL );
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSDA );
							
	
	for(wClockCnt = 0; wClockCnt < 8; wClockCnt++)
	{
		if(Data & (BYTE)wBitPointer)
		{
            AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSDA );
		}
		else
		{
            AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSDA );
		}
		DELAY(50);
        AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSCL );
		DELAY(30);
        AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSCL );
        DELAY(20);
		wBitPointer >>= 1;
	}
    AT91F_PIO_CfgInput( I2C_PIO_BASE, BIT_I2CSDA );	//data in
	DELAY( 20 );
    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSCL );
	DELAY( 20 );
	while( AT91F_PIO_IsInputSet(I2C_PIO_BASE, BIT_I2CSDA) )
	{
   		Count++;
   		if(Count == 1000L)
   		{
   			// stop
			Stop_I2C();
   			return FALSE;
   		}
		DELAY( 1 );
   	}
	DELAY( 20 );
    AT91F_PIO_ClearOutput( I2C_PIO_BASE, BIT_I2CSCL );
	DELAY(50 );
	return TRUE;
}





STATIC VOID
HW_WriteRegisters( BYTE Val, BYTE Reg, BYTE nReg )
{
//	WriteL3Addr(Reg);
//    WriteBIT_AUDIO_L3DAT( (BYTE)Val, 0);
	BOOL ret1,ret2,ret3;

	Start_I2C();
	ret1 = Write_I2C_Data( 0x34 );
	ret2 = Write_I2C_Data( Reg );
	ret3 = Write_I2C_Data( Val );
	Stop_I2C();
	//DEBUGMSG( 1,"%s ret (%x , %x , %x)\n",__FUNCTION__,ret1,ret2,ret3 );
}

//======================================================================================
#if 0

STATIC
VOID TWI_Write(BYTE DevAddr, UCHAR RegAddr, BYTE *Data, UINT32 DataCnt, BYTE Size)
{
	AT91PS_TWI	pTWI = (AT91PS_TWI)AT91C_BASE_TWI0;
	UINT32		timeout = 0;
	
    /* Set slave address and number of internal address bytes. */
    pTWI->TWI_MMR = 0;
    pTWI->TWI_MMR = (Size << 8) | (DevAddr << 16);
	
    /* Set internal address bytes. */
    pTWI->TWI_IADR = 0;
    pTWI->TWI_IADR = RegAddr;
	/* Write first byte to send.*/
	pTWI->TWI_THR = *Data;
	Data++;
	DataCnt--;
	/* Send all bytes */
	while (DataCnt > 0)
	{
		timeout = 0;
		/* Wait before sending the next byte */
		while( !((pTWI->TWI_SR & AT91C_TWI_TXRDY_MASTER) == AT91C_TWI_TXRDY_MASTER) && (++timeout<TWITIMEOUTMAX) );
		
		if (timeout == TWITIMEOUTMAX)
		{
			DEBUGMSG( DGB_LVL_TWI,"%s(%d): TWID Timeout BS\n",__FUNCTION__, __LINE__);
		}
		
		pTWI->TWI_THR = *Data;
		Data++;
		DataCnt--;
		//Sleep(1);
	}
	pTWI->TWI_CR |= AT91C_TWI_STOP;
	timeout = 0;
	/* Wait for transfer to be complete */
	while( !((pTWI->TWI_SR & AT91C_TWI_TXCOMP_MASTER) == AT91C_TWI_TXCOMP_MASTER) && (++timeout<TWITIMEOUTMAX) );
	if (timeout == TWITIMEOUTMAX)
	{
		DEBUGMSG( DGB_LVL_TWI,"%s(%d): TWID Timeout TC2\n",__FUNCTION__, __LINE__);
	}
}

STATIC
VOID TWI_Read(BYTE DevAddr, UCHAR RegAddr, BYTE *Data, UINT32 DataCnt, BYTE Size)
{
	AT91PS_TWI	pTWI = (AT91PS_TWI)AT91C_BASE_TWI0;
	UINT32		timeout = 0;
	BYTE		btWriteBuffer[2];
    btWriteBuffer[0] = (BYTE)(RegAddr<<1);
  
    TWI_Write( DevAddr, RegAddr, btWriteBuffer, 1, 0);
	
	/* Set STOP signal if only one byte is sent*/
	if(1 == DataCnt)
		pTWI->TWI_CR |= AT91C_TWI_STOP;

	/* Set slave address and number of internal address bytes. */
	pTWI->TWI_MMR = 0;
    pTWI->TWI_MMR = (1 << 8) | AT91C_TWI_MREAD | (DevAddr << 16);

    /* Set internal address bytes */
    pTWI->TWI_IADR = 0;
    pTWI->TWI_IADR = RegAddr;

    /* Send START condition */
    pTWI->TWI_CR |= AT91C_TWI_START;
	/* Read all bytes, setting STOP before the last byte*/
	while (DataCnt > 0)
	{
		/* Set STOP signal if only one byte is sent*/
		if(1 == DataCnt)
			pTWI->TWI_CR |= AT91C_TWI_STOP;

		timeout = 0;
		/* Wait for byte then read and store it*/
		while( !((pTWI->TWI_SR & AT91C_TWI_RXRDY ) == AT91C_TWI_RXRDY ) && (++timeout<TWITIMEOUTMAX) );
		
		if (timeout == TWITIMEOUTMAX)
		{
			DEBUGMSG( DGB_LVL_TWI,"%s(%d): TWID Timeout BR\n",__FUNCTION__, __LINE__);
		}
		*Data = pTWI->TWI_RHR;
		Data++;
		DataCnt--;
	}
	timeout = 0;
		/* Wait for transfer to be complete */
	while( !((pTWI->TWI_SR & AT91C_TWI_TXCOMP_MASTER) == AT91C_TWI_TXCOMP_MASTER) && (++timeout<TWITIMEOUTMAX) );
	if (timeout == TWITIMEOUTMAX)
	{
		DEBUGMSG( DGB_LVL_TWI,"%s(%d): TWID Timeout TC\n",__FUNCTION__, __LINE__);
	}
}
#endif

//------------------------------------------------------------------------------------------------------------
//! \brief Quick Mute the output with out effecting the stored volume value
//!
//! \return TRUE indicates success. FALSE indicates failure
//!
//-------------------------------------------------------------------------------------------------------------
BOOL
WaveOutMute( BOOL bMute )
{
    m_bOutputMute = bMute;//AC 130220 FOR WM8960
	DEBUGMSG( DGB_LVL_TWI,"%s(%d): ====\n",__FUNCTION__, __LINE__);
    if (bMute)
    {
        CODEC_WriteRegister(WM8960_DACCTL1, (m_WM8580_SFR_Table[WM8960_DACCTL1] | (1<<3)));  //DAC Digital Soft Mute
    }
    else
    {
        CODEC_WriteRegister(WM8960_DACCTL1, (m_WM8580_SFR_Table[WM8960_DACCTL1] & ~(1<<3)));  //DAC Digital Soft Unmute
    }

    return TRUE;
}

//------------------------------------------------------------------------------------------------------------
//! \brief Quick Mute the input with out effecting the stored volume value
//!
//! \return TRUE indicates success. FALSE indicates failure
//!
//-------------------------------------------------------------------------------------------------------------
BOOL
WaveInMute( BOOL bMute )
{
	DEBUGMSG( DGB_LVL_TWI,"%s(%d): ====\n",__FUNCTION__, __LINE__);    
    m_bInputMute = bMute;
//    return m_InputDeviceContext.SetGain(bMute ? 0: m_dwInputGain);
    if(bMute)
    {
        CODEC_WriteRegister(WM8960_LINVOL, m_WM8580_SFR_Table[WM8960_LINVOL] | (1<<7));  // Analog Mute
    }
    else
    {
        CODEC_WriteRegister(WM8960_LINVOL, (m_WM8580_SFR_Table[WM8960_LINVOL] & ~(1<<7)) | (1<<8));  // Analog Unmute, IPVU must be set, Todo: volume recovery
    }

    return TRUE;
}


//------------------------------------------------------------------------------------------------------------
//! \brief Convert the Windows Volume to an AC 97 volume and set it
//!
//! \return TRUE indicates success. FALSE indicates failure
//!
//-------------------------------------------------------------------------------------------------------------

STATIC BOOL
WaveOutSetVolume( UINT32 nVolume )
{
    USHORT ucL3Vol, ucL3Vol2;
	
	ucL3Vol2=(USHORT) ((nVolume & 0x0000ffff)/256 )|0x100; //drop right channel volume
	ucL3Vol=(USHORT) (((nVolume & 0xffff0000)>>16)/256 )|0x100; //drop LEFT channel volume

	ucL3Vol2 +=(USHORT) ((0x1ff-ucL3Vol2)/2);  
	ucL3Vol +=(USHORT) ((0x1ff-ucL3Vol)/2);	

	DEBUGMSG( 1,"%s(%d): 1Vol(%x%x) \n",__FUNCTION__,__LINE__, ucL3Vol, ucL3Vol2 );

	
	CODEC_WriteRegister( WM8960_LDAC, m_WM8580_SFR_Table[WM8960_LDAC] | ucL3Vol );
//	CODEC_WriteRegister( WM8960_LDAC, ucL3Vol&0x00ff );
	CODEC_WriteRegister( WM8960_RDAC, m_WM8580_SFR_Table[WM8960_RDAC] | ucL3Vol2 );
//	CODEC_WriteRegister( WM8960_RDAC, ucL3Vol2&0x00ff );	//spker only left

//	DEBUGMSG( 1,"%s Vol(%x%x) , ",__FUNCTION__, ucL3Vol, ucL3Vol2 );

    ucL3Vol2=(USHORT) ((nVolume & 0x0000ffff)/512 )|0x100; //drop right channel volume
    ucL3Vol=(USHORT) (((nVolume & 0xffff0000)>>16)/512 )|0x100; //drop right channel volume


	if( ucL3Vol < 0x130 )
		ucL3Vol = 0x130;

	if( ucL3Vol2 < 0x130 )
		ucL3Vol2 = 0x130;

	DEBUGMSG( 1,"%s(%d): 2Vol(%x%x) \n",__FUNCTION__, __LINE__, ucL3Vol, ucL3Vol2 );

	// Headphone Volume
	CODEC_WriteRegister( WM8960_LOUT1, m_WM8580_SFR_Table[WM8960_LOUT1] | 0x17f );
	//CODEC_WriteRegister( WM8960_LOUT1, m_WM8580_SFR_Table[WM8960_LOUT1] & ~0x7f );
	CODEC_WriteRegister( WM8960_ROUT1, m_WM8580_SFR_Table[WM8960_ROUT1] | 0x179 );
	//CODEC_WriteRegister( WM8960_ROUT1, m_WM8580_SFR_Table[WM8960_ROUT1] & ~0x7f );
	
	//CODEC_WriteRegister( WM8960_ROUT1, 0x17f );

	CODEC_WriteRegister( WM8960_LOUT2, m_WM8580_SFR_Table[WM8960_LOUT2] | 0x17f );
	//CODEC_WriteRegister( WM8960_ROUT2, m_WM8580_SFR_Table[WM8960_ROUT2] | ucL3Vol2 );
	CODEC_WriteRegister( WM8960_ROUT2, m_WM8580_SFR_Table[WM8960_ROUT2] | 0x17f );


    v_nVolume = nVolume;

    return TRUE;    
}
//------------------------------------------------------------------------------------------------------------
//! \brief Convert the Windows Volume to an AC 97 volume and set it
//!
//! \return TRUE indicates success. FALSE indicates failure
//!
//-------------------------------------------------------------------------------------------------------------

STATIC BOOL
WaveInSetVolume( UINT32 uVolume )
{
//    CODEC_SEND_CMD(0x00,nVolume);    // Llinein register control
//    CODEC_SEND_CMD(0x02,nVolume);    // Rlinein register control

    return TRUE;
    
}


//------------------------------------------------------------------------------------------------------------
//! \brief Write the sample rate value to the ac97 codec
//-------------------------------------------------------------------------------------------------------------

//struct _coeff_div {
//    UINT32  Clock;
//    UINT32  SampleRate;
//    UINT16  FS;
//    UINT8   SR:5;
//    UINT8   USB:1;
//};
//
///* codec hifi mclk clock divider coefficients */
//static const struct _coeff_div coeff_div[] = 
//{
//    /* 8k */
//    {12288000, 8000, 1536, 0x6, 0x0},
//    {11289600, 8000, 1408, 0x16, 0x0},
//    {18432000, 8000, 2304, 0x7, 0x0},
//    {16934400, 8000, 2112, 0x17, 0x0},
//    {12000000, 8000, 1500, 0x6, 0x1},
//
//    /* 11.025k */
//    {11289600, 11025, 1024, 0x18, 0x0},
//    {16934400, 11025, 1536, 0x19, 0x0},
//    {12000000, 11025, 1088, 0x19, 0x1},
//
//    /* 16k */
//    {12288000, 16000, 768, 0xa, 0x0},
//    {18432000, 16000, 1152, 0xb, 0x0},
//    {12000000, 16000, 750, 0xa, 0x1},
//
//    /* 22.05k */
//    {11289600, 22050, 512, 0x1a, 0x0},
//    {16934400, 22050, 768, 0x1b, 0x0},
//    {12000000, 22050, 544, 0x1b, 0x1},
//
//    /* 32k */
//    {12288000, 32000, 384, 0xc, 0x0},
//    {18432000, 32000, 576, 0xd, 0x0},
//    {12000000, 32000, 375, 0xa, 0x1},
//
//    /* 44.1k */
//    {11289600, 44100, 256, 0x10, 0x0},
//    {16934400, 44100, 384, 0x11, 0x0},
//    {12000000, 44100, 272, 0x11, 0x1},
//
//    /* 48k */
//    {12288000, 48000, 256, 0x0, 0x0},
//    {18432000, 48000, 384, 0x1, 0x0},
//    {12000000, 48000, 250, 0x0, 0x1},
//
//    /* 88.2k */
//    {11289600, 88200, 128, 0x1e, 0x0},
//    {16934400, 88200, 192, 0x1f, 0x0},
//    {12000000, 88200, 136, 0x1f, 0x1},
//
//    /* 96k */
//    {12288000, 96000, 128, 0xe, 0x0},
//    {18432000, 96000, 192, 0xf, 0x0},
//    {12000000, 96000, 125, 0xe, 0x1},
//};
//

STATIC BOOL
AudioSetSampleRate(WAPI_INOUT apidir, UINT16 SampleRate )
{
    BOOL    bRet = FALSE;
//    UINT16  InterfaceReg, SampleRateReg;
//    INT32   coeff;

    switch(SampleRate)
    {
    case 8000:
    case 11025:
    case 16000:
    case 22050:
    case 32000:
    case 44100:
    case 48000:
        bRet=TRUE;
        break;

    default:
        bRet=FALSE;
        return bRet;
        
        
    }
    
    return bRet;
}


//------------------------------------------------------------------------------------------------------------
//! \brief power up or power down a dac, adc, or other devices
//-------------------------------------------------------------------------------------------------------------
STATIC VOID
AudioPowerDownUnit( UINT16 Unit, BOOL bShutDown )
{
////    if( bShutDown )
////    {
////        g_pIOPregs->GPGDAT &= ~( (0x1<<(AUDIO_SPKEN)) );    // low - disable
////        g_pIOPregs->GPADAT &= ~( (0x1<<(AMP_POWER_ON)) );   // low - disable
////    }
////    else
//    {
//        g_pIOPregs->GPGDAT &= ~( (0x1<<(AUDIO_SPKEN)) );    // low - enable
//        g_pIOPregs->GPADAT |=  ( (0x1<<(AMP_POWER_ON)) );   // high - enable
//    }
//
////    register UINT16 usiMask,usiMaskedValue;
////    UINT16          PowerDownCtrlStat=0;
////    
////    if( !ShadowReadAC97( POWERDOWN_CTRL_STAT, &PowerDownCtrlStat)  )  // Enable Variable Rate Audio
////    {
////kprintf( "-- read of PwrDnCtrl failed \n");
////    }    
////
////    usiMask = Unit;
////    if (bShutDown)
////    {
////        usiMaskedValue = Unit;
////    }
////    else
////    {
////        usiMaskedValue = 0;
////    }
////
////    if ((PowerDownCtrlStat & usiMask) != usiMaskedValue)
////    {
////        PowerDownCtrlStat = (PowerDownCtrlStat & ~usiMask) | usiMaskedValue;
////        
////        if( !ShadowWriteAC97( POWERDOWN_CTRL_STAT, PowerDownCtrlStat)  )  // Enable Variable Rate Audio
////        {
////            kprintf("-- write of PwrDnCtrl failed \n");
////        }
////    }
    
}

//------------------------------------------------------------------------------------------------------------
//! \brief Routine for powering on Audio hardware.  Reinit the codec
//!
//! \note This may be called from within power handler, so should not make any system calls.
//-------------------------------------------------------------------------------------------------------------
STATIC BOOL
AudioPowerOn( VOID )
{
//    UINT16      Reg;
//
//
//    // WM9714 Codec Power On
//    CODEC_PowerOn();
//
//    // Enable Clock for AC97
//    g_pCMUCLKReg->PCLK_GATE |= (1<<14);
//
//    InitAC97();
//
//    InitCodec();
//
//    CODEC_MuteControl(DMA_CH_OUT|DMA_CH_IN, TRUE);



    return (TRUE);
}

//------------------------------------------------------------------------------------------------------------
//! \brief Routine for powering down Audio hardware.
//-------------------------------------------------------------------------------------------------------------
STATIC VOID
AudioPowerOff( VOID )
{
  
//    WavHal_WaveOutStop();
//    WavHal_WaveInStop(NULL);
//    
//
//    WaveOutMute(TRUE);
//    WaveInMute(TRUE);      
//    
//    CODEC_MuteControl(DMA_CH_OUT|DMA_CH_IN, TRUE);
//    CODEC_PowerControl();
//
//    // Disable Clock for AC97
//    g_pCMUCLKReg->PCLK_GATE &= ~(1<<14);
//
//    // WM9714 Codec Power On
//    CODEC_PowerOff();
//
//    AudioPowerDownUnit(WAPI_OUT, TRUE);
//    AudioPowerDownUnit(WAPI_IN, TRUE);
    
}

STATIC
VOID SSC_Init(UINT32 BitRate, UINT32 MasterClock)
{
	AT91PS_PMC pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
	AT91PS_SSC pSSC = (AT91PS_SSC)AT91C_BASE_SSC0;
	AT91PS_TWI pTWI = (AT91PS_TWI)AT91C_BASE_TWI0;

	UINT32 dwCkDiv = 0 ;
    UINT32 dwClDiv ;
    UINT32 dwOk = 0 ;
	
    /* Enable SSC peripheral clock */
    pPMC->PMC_PCER1 = 1 << (SAMA5D3_ID_SSC0 - 32);
	//AUDIO MCLK
	pPMC->PMC_PCKR[1] = AT91C_PMC_CSS_MAIN_CLK;
	pPMC->PMC_SCER = AT91C_PMC_PCK1;
	while ((pPMC->PMC_SR & AT91C_PMC_PCK1RDY) == 0)
	{
		Sleep(10);
	}
	
    /* Reset, disable receiver & transmitter */
    pSSC->SSC_CR = SSC_CR_RXDIS | SSC_CR_TXDIS | SSC_CR_SWRST;

	/* Configure clock frequency */
    if (BitRate == 0)
	{
		pSSC->SSC_CMR = 0;
    }
    else
	{
		pSSC->SSC_CMR = MasterClock / (2 * BitRate);
		Sleep(10);
    }

    pSSC->SSC_RCMR = I2S_MASTER_RX_SETTING(BITS_BY_SLOT, SLOT_BY_FRAME);
    pSSC->SSC_RFMR = I2S_MASTER_RX_FRM_SETTING(BITS_BY_SLOT, SLOT_BY_FRAME);
	pSSC->SSC_TCMR = I2S_MASTER_TX_SETTING(BITS_BY_SLOT, SLOT_BY_FRAME);
	pSSC->SSC_TFMR = I2S_MASTER_TX_FRM_SETTING(BITS_BY_SLOT, SLOT_BY_FRAME);
#if 0
	//TWI0 Init(I2C)
	pPMC->PMC_PCER = 1 << AT91C_ID_TWI0;

    /* SVEN: TWI Slave Mode Enabled */
    //pTWI->TWI_CR = AT91C_TWI_SVEN ;
    /* Reset the TWI */
    pTWI->TWI_CR = AT91C_TWI_SWRST ;
    pTWI->TWI_RHR ;
    /* TWI Slave Mode Disabled, TWI Master Mode Disabled. */
    pTWI->TWI_CR = AT91C_TWI_SVDIS ;
    pTWI->TWI_CR = AT91C_TWI_MSDIS ;
    /* Set master mode */
    pTWI->TWI_CR = AT91C_TWI_MSEN ;

    /* Configure clock */
    while ( !dwOk )
    {
        dwClDiv = ((SSC_MCK / (2 * TWI_CLOCK)) - 4) / (1<<dwCkDiv) ;

        if ( dwClDiv <= 255 )
        {
            dwOk = 1 ;
        }
        else
        {
            dwCkDiv++ ;
        }
    }

    pTWI->TWI_CWGR = 0 ;
    pTWI->TWI_CWGR = (dwCkDiv << 16) | (dwClDiv << 8) | dwClDiv ;
#endif
}



STATIC VOID
CODEC_WriteRegister(UCHAR Reg, USHORT Val)
{
    // WM8580
    // SFR[B15..B9], DATA[B8..B0]
    // +------------------------+
    // | B15...B9 B8.........B0 |
    // +------------------------+
    //
    BYTE btWriteBuffer[2];

    m_WM8580_SFR_Table[Reg] = Val;

    //DEBUGMSG( DBG_LVL_WAV,"%s(%d): (0x%02x, 0x%x)\n\r",__FUNCTION__, __LINE__, Reg, Val);

    btWriteBuffer[0] = (BYTE)(Reg<<1)|(BYTE)((Val>>8)&0x01);
    btWriteBuffer[1] = (BYTE)(Val&0xFF);
    Reg = (Reg<<1) | ((Val>>8)&1);
   // Reg = Reg;//(Reg<<1) & 0xfe ;
    Val = Val &0xFF;

    HW_WriteRegisters( Val, Reg, 1 );
//    TWI_Write( I2C_CODEC_ID, Reg, btWriteBuffer, 2, 0);
//	Sleep(60);
    //DEBUGMSG( DBG_LVL_WAV,"%s(%d): (0x%02x, 0x%x)\n\r",__FUNCTION__, __LINE__, Reg, Val);
}

STATIC USHORT
CODEC_ReadRegister(UCHAR Reg)
{
    USHORT RegValue = 0;
    //-----------------------------------------------------------------------------------------------------
    // !!!!! Caution !!!!!
    // WM8580 support write operation only.
    // I write sfr data value onto internal table(m_WM8580_SFR_Table), on every write operation for backup.
    // And To support read operation, I just return some value from internal table(m_WM8580_SFR_Table).
    //-----------------------------------------------------------------------------------------------------

    RegValue = m_WM8580_SFR_Table[(int)Reg];
	//TWI_Read(I2C_CODEC_ID, Reg, btReadBuffer, 2, 0);
    //DEBUGMSG( 1,"%s(%d): Reg (0x%X)  btReadBuffer[0] = 0x%X, btReadBuffer[1]=0x%X\n\r",__FUNCTION__, __LINE__, Reg, btReadBuffer[0],btReadBuffer[1]);

	return RegValue;
}


STATIC BOOL
CODEC_Init( VOID )
{
    int i;
	USHORT val;
	BYTE buf[2];

    AT91F_PIO_SetOutput( I2C_PIO_BASE, BIT_I2CSCL );	//start condition : BIT_L3M=H, BIT_L3C=H

    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSCL );
    AT91F_PIO_CfgOutput( I2C_PIO_BASE, BIT_I2CSDA );

	memset(m_WM8580_SFR_Table, 0x00, WM8580_MAX_REGISTER_COUNT*2);
	
	/****** L3 Interface ******/
    // Default setting value
#if 1	//WM8960
	for(i=0; i<(WM8960_INIT_SIZE); i++)
	{
		if(WM8960_InitValue[i].addr == 0xff) 
		{
			if (WM8960_InitValue[i].data == 0x19)
					Sleep(400);//delayLoop(400);
			else 
					Sleep(200);//delayLoop(100);
		}
		else
		{
			CODEC_WriteRegister( WM8960_InitValue[i].addr, WM8960_InitValue[i].data );
			DEBUGMSG(0,"%s(%d): addr 0x%x, data 0x%x\r\n", __FUNCTION__, __LINE__, WM8960_InitValue[i].addr, WM8960_InitValue[i].data);
			Sleep(10);
		}
    }
	Sleep(100);
	WaveOutSetVolume(0xffffffff);
	//Sleep(50);
#if 0
	for(i=0; i<(WM8960_POWERUP_SIZE); i++)
	{
		if(WM8960_Power_Up[i].addr == 0xff) 
		{
			if (WM8960_Power_Up[i].data == 0x19)
					Sleep(400);//delayLoop(400);
			else 
					Sleep(100);//delayLoop(100);
		}
		else
		{
			CODEC_WriteRegister( WM8960_Power_Up[i].addr, WM8960_Power_Up[i].data );
			DEBUGMSG(0,"%s(%d): addr 0x%x, data 0x%x\r\n", __FUNCTION__, __LINE__, WM8960_Power_Up[i].addr, WM8960_Power_Up[i].data);
			Sleep(10);
		}
    }
#endif
#endif
#if 0	//WM8731
	CODEC_WriteRegister(WM8731_REG_RESET, 0);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_LEFT_HPOUT, 0x60);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_RIGHT_HPOUT, 0x60);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_ANALOGUE_PATH_CTRL, 0x12);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_DIGITAL_PATH_CTRL, 0);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_DA_INTERFACE_FORMAT, (WM8731_FORMAT_I2S | WM8731_MS_BIT));
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_SAMPLECTRL, 0);
	Sleep(10);
	CODEC_WriteRegister(WM8731_REG_PWDOWN_CTRL, 0x07);
	Sleep(10);

	CODEC_WriteRegister(WM8731_REG_ACTIVE_CTRL, 0x01);
	Sleep(10);
#if 0
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_LEFT_HPOUT);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_RIGHT_HPOUT);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_ANALOGUE_PATH_CTRL);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_DIGITAL_PATH_CTRL);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_DA_INTERFACE_FORMAT);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_SAMPLECTRL);
			Sleep(1500);
	CODEC_ReadRegister(WM8731_REG_PWDOWN_CTRL);
	CODEC_ReadRegister(WM8731_REG_ACTIVE_CTRL);
	#endif
#endif
    return TRUE;
}



STATIC VOID
CODEC_PowerOn( VOID )
{
    DEBUGMSG( DBG_LVL_WAV,"%s(%d): ====\n",__FUNCTION__, __LINE__);

    // Codec Power is no controllerable in SMDK6410

}

STATIC VOID
CODEC_PowerOff( VOID )
{
    DEBUGMSG( DBG_LVL_WAV,"%s(%d): ======= \n",__FUNCTION__, __LINE__);

    // Codec Power is no controllerable in SMDK6410

}



STATIC BOOL
CODEC_PowerControl( VOID )
{
/*
    if( m_InputDMARunning & m_OutputDMARunning )
    {
        CODEC_WriteRegister(WM8580_PWRDN1, WM8580_ALL_POWER_ON);                    // ADC, DAC power up
    }
    else if( m_InputDMARunning )
    {
        CODEC_WriteRegister(WM8580_PWRDN1, WM8580_PWRDN1_ADCPD_ENABLE|WM8580_PWRDN1_DACPD_ALL_DISABLE);        // ADC power up, DAC power down
    }
    else if( m_OutputDMARunning )
    {
        CODEC_WriteRegister(WM8580_PWRDN1, WM8580_PWRDN1_DACPD_ALL_ENABLE|WM8580_PWRDN1_ADCPD_DISABLE);   // DAC power up, ADC power down
    }
    else
    {
        CODEC_WriteRegister(WM8580_PWRDN1, WM8580_PWRDN1_ADCPD_DISABLE|WM8580_PWRDN1_DACPD_ALL_DISABLE ); // ADC/DAC power down
    }
*/
    return(TRUE);
}


STATIC BOOL
CODEC_MuteControl( DWORD channel, BOOL bMute )
{
	DEBUGMSG( DGB_LVL_TWI,"%s(%d): channel = %d, bMute = %d\n",__FUNCTION__, __LINE__,channel,bMute);//AC 130220 FOR WM8960
    if((channel & DMA_CH_OUT ))// && !m_OutputDMARunning )
    {
        if (bMute)
        {
			CODEC_WriteRegister(WM8960_DACCTL1, m_WM8580_SFR_Table[WM8960_DACCTL1] | (1<<7));  //DAC Digital Soft Mute
        }
        else
        {
            CODEC_WriteRegister(WM8960_DACCTL1, m_WM8580_SFR_Table[WM8960_DACCTL1] & ~(1<<7));  //DAC Digital Soft Unmute
        }
    }

    if( (channel & DMA_CH_IN))// && !m_InputDMARunning )
    {
        if(bMute)
        {
            CODEC_WriteRegister(WM8960_LINVOL, m_WM8580_SFR_Table[WM8960_LINVOL] | (1<<7));  // Analog Mute
        }
        else
        {
            CODEC_WriteRegister(WM8960_LINVOL, (m_WM8580_SFR_Table[WM8960_LINVOL] & ~(1<<7)) | (1<<8));  // Analog Unmute, IPVU must be set, Todo: volume recovery
        }
    }

    return TRUE;
}


STATIC VOID
CODEC_SetSpeakerEnable( BOOL bEnable )
{
    // Code to turn speaker on/off here
//    USHORT CodecReg;
    DEBUGMSG( DGB_LVL_TWI,"%s(%d): ====\n",__FUNCTION__, __LINE__);
    if(bEnable == TRUE)
    {
        CODEC_WriteRegister(WM8960_CLASSD1, m_WM8580_SFR_Table[WM8960_CLASSD1] | (3<<6));
    }
    else
    {	 
        CODEC_WriteRegister(WM8960_CLASSD1, m_WM8580_SFR_Table[WM8960_CLASSD1] & ~(3<<6));
    }	

    return;
}

//------------------------------------------------------------------------------------------------------------
//! \brief initializes DMA
//-------------------------------------------------------------------------------------------------------------

STATIC BOOL  
DMA_Init(WAPI_INOUT wapidir )
{
    BOOL    bRet;
	UINT32  iController;
	UINT32  dwCfg;
	
    switch(wapidir)
    {
    case WAPI_IN:
    {
        DEBUGMSG( DBG_LVL_WAV,"%s(%d): WAPI_IN\n",__FUNCTION__, __LINE__);

        nDmaChannelRx = DMAD_AllocateChannel(SAMA5D3_ID_SSC0, DMAD_TRANSFER_MEMORY);
		if (nDmaChannelRx == DMAD_ALLOC_FAILED)
		{
			DEBUGMSG(DBG_LVL_WAV, "%s (%d): DMA channel allocat error\n",__FUNCTION__,__LINE__);
			return FALSE;
		}
		
        {
            SYSINTR_REQUEST     SysIntrRequest;
        
            // request sysintr    
            SysIntrRequest.nHalIrq =  nDmaChannelRx + IRQ_VIR_DMA_START;
            SysIntrRequest.BitMap = 1 << DMA_CH_IN; //1<<0;
            SysIntrRequest.Flag = SYSINTR_REQ_FLAG_MULTIPLE;

            if( !KernelIoControl( IOCTRL_HAL_SYSINTR_REQUEST, &SysIntrRequest, sizeof( SYSINTR_REQUEST ), &snSysIntr, sizeof(UINT32) ) )
            {
                DEBUGMSG( 1,"%s (%d): snSysIntr = %d\n", __FUNCTION__, __LINE__, snSysIntr );
            }
        }
		/* Configure DMA RX channel */
		iController = (nDmaChannelRx >> 3);
		dwCfg = 0
			   | DMAC_CFG_SRC_PER(DMAIF_Get_ChannelNumber( iController, SAMA5D3_ID_SSC0, DMAD_TRANSFER_RX ))           
			   | DMAC_CFG_SRC_H2SEL
			   | DMAC_CFG_SOD_DISABLE
			   | DMAC_CFG_FIFOCFG_ALAP_CFG;
		DMAD_PrepareChannel( nDmaChannelRx, dwCfg );
		//DEBUGMSG( 1,"%s (%d): snSysIntr = %d\n", __FUNCTION__, __LINE__, snSysIntr );
		DEBUGMSG( 1,"%s (%d): DmaControler_WaveRx = %X, DmaChannl_WaveRx = %X\n", __FUNCTION__, __LINE__, (nDmaChannelRx >> 3), (nDmaChannelRx & 0x07) );


        break;
    }    
    case WAPI_OUT:
    {
        DEBUGMSG( DBG_LVL_WAV,"%s(%d): WAPI_OUT\n\r",__FUNCTION__, __LINE__);

        nDmaChannelTx = DMAD_AllocateChannel(DMAD_TRANSFER_MEMORY, SAMA5D3_ID_SSC0);
		DEBUGMSG( 1,"%s (%d): nDmaChannelTx = %X\n", __FUNCTION__, __LINE__, (nDmaChannelTx));
		DEBUGMSG( 1,"%s (%d): DmaControler = %X, DmaChannl = %X\n", __FUNCTION__, __LINE__, (nDmaChannelTx >> 3), ( nDmaChannelTx & 0x07) );
        {
            SYSINTR_REQUEST     SysIntrRequest;
        
            // request sysintr    
            SysIntrRequest.nHalIrq = ( nDmaChannelTx & 0x07) + (nDmaChannelTx >> 3)*8 + IRQ_VIR_DMA_START;
            SysIntrRequest.BitMap = 1 << DMA_CH_OUT;//1<<1;
            SysIntrRequest.Flag = SYSINTR_REQ_FLAG_STATIC;

            if( !KernelIoControl( IOCTRL_HAL_SYSINTR_REQUEST, &SysIntrRequest, sizeof( SYSINTR_REQUEST ), &snSysIntr, sizeof(UINT32) ) )
            {
                DEBUGMSG( 1,"%s (%d): snSysIntr = %d\n", __FUNCTION__, __LINE__, snSysIntr );
            }
        }

		/* Configure DMA TX channel */
		iController = (nDmaChannelTx >> 3);
		dwCfg = 0           
			   | DMAC_CFG_DST_PER(DMAIF_Get_ChannelNumber( iController, SAMA5D3_ID_SSC0, DMAD_TRANSFER_TX ))
			   | DMAC_CFG_DST_H2SEL
			   | DMAC_CFG_SOD_DISABLE
			   | DMAC_CFG_FIFOCFG_HALF_CFG;
			   //| DMAC_CFG_FIFOCFG_ALAP_CFG;
		DMAD_PrepareChannel( nDmaChannelTx, dwCfg );

		//DEBUGMSG( 1,"%s (%d): snSysIntr = %d\n", __FUNCTION__, __LINE__, snSysIntr );

        break;
    }    
    default:
        return FALSE;
    }
    
    return(TRUE);
}

STATIC UINT32 DMA_TXBuffer;
STATIC UINT32 DMA_TXBuffer2;
STATIC UINT32 DMA_RXBuffer;
STATIC UINT32 DMA_RXBuffer2;
STATIC UINT32 DMA_TXBuf_Size;
STATIC UINT32 DMA_TXBuf2_Size;
STATIC UINT32 DMA_RXBuf_Size;
STATIC UINT32 DMA_RXBuf2_Size;
#define MAX_DESC_NUM 2
HDMA_TRANSFER_DESC  TranDesc[MAX_DESC_NUM];

//------------------------------------------------------------------------------------------------------------
//! \brief stops PDC (DMA) transfer
//-------------------------------------------------------------------------------------------------------------
STATIC BOOL  
DMA_Start(WAPI_INOUT wapidir)
{
	AT91PS_SSC pSSC = (AT91PS_SSC)AT91C_BASE_SSC0;
	//HDMA_TRANSFER_DESC  *TranDesc;
    switch(wapidir)
    {
    case WAPI_IN:
        if(m_InputDMARunning)
            break;

        //----- 1. Initialize our buffer counters -----
        m_InputDMARunning=TRUE;


//        m_nInByte[INPUT_DMA_PAGE0] = 0;
//        m_nInByte[INPUT_DMA_BUFFER1] = 0;

//        m_nInputBufferInUse = INPUT_DMA_PAGE0;    // Start DMA with Buffer 0
        m_InputDMAStatus = (DMA_DONEA | DMA_DONEB) & ~DMA_BIU;

        CODEC_PowerControl();                       // Turn On Channel
        CODEC_MuteControl(DMA_CH_IN, FALSE);        // Unmute Input Channel

		TranDesc[0].SrcAddr = (UINT32)&pSSC->SSC_RHR;
		TranDesc[0].DstAddr = (UINT32)DMA_RXBuffer;
		TranDesc[0].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_RXBuf_Size/2) |
							 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
							 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
		TranDesc[0].CtrlB   = DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
							 DMAC_CTRLB_FC_PER2MEM_DMA_FC |
							 DMAC_CTRLB_SRC_INCR_FIXED |
							 DMAC_CTRLB_DST_INCR_INCREMENTING;
		TranDesc[0].DscAddr = (UINT32)&TranDesc[1] + DMA_ADDR_OFFSET;
		CacheRangeFlush(DMA_RXBuffer - DMA_ADDR_OFFSET, DMA_RXBuf_Size, CACHE_SYNC_DISCARD);
		if(DMA_RXBuffer2)
		{
			TranDesc[1].SrcAddr = (UINT32)&pSSC->SSC_RHR;
			TranDesc[1].DstAddr = (UINT32)DMA_RXBuffer2;
			TranDesc[1].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_RXBuf2_Size/2) |
								 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
								 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
			TranDesc[1].CtrlB   = DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
								 DMAC_CTRLB_FC_PER2MEM_DMA_FC |
								 DMAC_CTRLB_SRC_INCR_FIXED |
								 DMAC_CTRLB_DST_INCR_INCREMENTING;
			TranDesc[1].DscAddr = 0;
			CacheRangeFlush(DMA_RXBuffer2 - DMA_ADDR_OFFSET, DMA_RXBuf2_Size, CACHE_SYNC_DISCARD);
		}
		CacheRangeFlush(TranDesc, sizeof(HDMA_TRANSFER_DESC)*2, CACHE_SYNC_WRITEBACK);
		DMAD_PrepareMultiTransfer(nDmaChannelRx, (HDMA_TRANSFER_DESC *)((UINT32)TranDesc + DMA_ADDR_OFFSET));
		//DMAD_PrepareSingleTransfer( nDmaChannelRx, &TranDesc);
		DMAD_StartTransfer( nDmaChannelRx);
		pSSC->SSC_CR = SSC_CR_RXEN;
		break;
        
    case WAPI_OUT:    
   
        //----- 1. Initialize our buffer counters -----

		{
			if(m_OutputDMARunning)
				break;

			m_OutputDMARunning = TRUE;

#if 0	//over 3 multi decripter 
			UINT32 i =0;
			for(i=0;i<MAX_DESC_NUM;i++)
			{
				if((i%2) == 0)
				{
					TranDesc[i].SrcAddr = (UINT32)DMA_TXBuffer;
					TranDesc[i].DstAddr = (UINT32)&pSSC->SSC_THR;
					TranDesc[i].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_TXBuf_Size/2) |
										 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
										 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
					TranDesc[i].CtrlB   = 0|//DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
										 DMAC_CTRLB_FC_MEM2PER_DMA_FC |
										 DMAC_CTRLB_SRC_INCR_INCREMENTING |
										 DMAC_CTRLB_DST_INCR_FIXED;
					TranDesc[i].DscAddr =(UINT32)&TranDesc[i+1] + DMA_ADDR_OFFSET;

				}
				else if((i%2) == 1)
				{

					TranDesc[i].SrcAddr = (UINT32)DMA_TXBuffer2;
					TranDesc[i].DstAddr = (UINT32)&pSSC->SSC_THR;
					TranDesc[i].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_TXBuf2_Size/2) |
									 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
									 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
					TranDesc[i].CtrlB   = 0|//DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
									 DMAC_CTRLB_FC_MEM2PER_DMA_FC |
									 DMAC_CTRLB_SRC_INCR_INCREMENTING |
									 DMAC_CTRLB_DST_INCR_FIXED;
					TranDesc[i].DscAddr = (UINT32)&TranDesc[i+1] + DMA_ADDR_OFFSET;
				}
				
				if(i  == (MAX_DESC_NUM -1))
				   TranDesc[i].DscAddr = (UINT32)&TranDesc[0] + DMA_ADDR_OFFSET;

				CacheRangeFlush(&TranDesc[i], sizeof(HDMA_TRANSFER_DESC), CACHE_SYNC_WRITEBACK);
				CacheRangeFlush(DMA_TXBuffer - DMA_ADDR_OFFSET, DMA_TXBuf_Size, CACHE_SYNC_WRITEBACK);
				CacheRangeFlush(DMA_TXBuffer2 - DMA_ADDR_OFFSET, DMA_TXBuf2_Size, CACHE_SYNC_WRITEBACK);
				
			}
#else//2 multi decripter 
			TranDesc[0].SrcAddr = (UINT32)DMA_TXBuffer;
			TranDesc[0].DstAddr = (UINT32)&pSSC->SSC_THR;
			TranDesc[0].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_TXBuf_Size/2) |
								 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
								 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
			TranDesc[0].CtrlB   = 0|//DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
								 DMAC_CTRLB_FC_MEM2PER_DMA_FC |
								 DMAC_CTRLB_SRC_INCR_INCREMENTING |
								 DMAC_CTRLB_DST_INCR_FIXED /*|
								 DMAC_CTRLB_AUTO*/
								 ;
			TranDesc[0].DscAddr =(UINT32)&TranDesc[1] + DMA_ADDR_OFFSET;

			//CacheRangeFlush(DMA_TXBuffer - DMA_ADDR_OFFSET, DMA_TXBuf_Size, CACHE_SYNC_WRITEBACK);
			CacheRangeFlush(&TranDesc[0], sizeof(HDMA_TRANSFER_DESC), CACHE_SYNC_WRITEBACK);

			TranDesc[1].SrcAddr = (UINT32)DMA_TXBuffer2;
			TranDesc[1].DstAddr = (UINT32)&pSSC->SSC_THR;
			TranDesc[1].CtrlA   = DMAC_CTRLA_BTSIZE(DMA_TXBuf2_Size/2) |
							 DMAC_CTRLA_SRC_WIDTH_HALF_WORD |
							 DMAC_CTRLA_DST_WIDTH_HALF_WORD;
			TranDesc[1].CtrlB   = 0|//DMAC_CTRLB_SRC_DSCR | DMAC_CTRLB_DST_DSCR |
							 DMAC_CTRLB_FC_MEM2PER_DMA_FC |
							 DMAC_CTRLB_SRC_INCR_INCREMENTING |
							 DMAC_CTRLB_DST_INCR_FIXED /*|
							 DMAC_CTRLB_AUTO*/
							 ;
			TranDesc[1].DscAddr = (UINT32)&TranDesc[0] + DMA_ADDR_OFFSET;
			//CacheRangeFlush(DMA_TXBuffer2 - DMA_ADDR_OFFSET, DMA_TXBuf2_Size, CACHE_SYNC_WRITEBACK);
			CacheRangeFlush(&TranDesc[1], sizeof(HDMA_TRANSFER_DESC), CACHE_SYNC_WRITEBACK);
#endif
    
            m_OutputDMAStatus = (DMA_DONEA | DMA_DONEB) & ~DMA_BIU;
            //OutputTransferred = TransferOutputBuffer(m_OutputDMAStatus);
			CODEC_PowerControl();   
            // Turn Output Channel
			CODEC_MuteControl(DMA_CH_OUT, FALSE);    // nmute Output Channel

			DMAD_PrepareMultiTransfer(nDmaChannelTx, (HDMA_TRANSFER_DESC *)((UINT32)&TranDesc[0] + DMA_ADDR_OFFSET));

			//DMAD_PrepareSingleTransfer( nDmaChannelTx, &TranDesc[0]);
			DMAD_StartTransfer( nDmaChannelTx);
			pSSC->SSC_CR = SSC_CR_TXEN;
		}
        break;
        
    default:
//START_ERROR:
        return FALSE;
    }
    
    return(TRUE);
}

STATIC BOOL  
DMA_Update(WAPI_INOUT wapidir)
{
	
}

//------------------------------------------------------------------------------------------------------------
//! \brief stops PDC (DMA) transfer
//-------------------------------------------------------------------------------------------------------------
STATIC BOOL  
DMA_Stop(WAPI_INOUT wapidir)
{
	AT91PS_SSC pSSC = (AT91PS_SSC)AT91C_BASE_SSC0;
    switch(wapidir)
    {
    case WAPI_IN:
        //----- 1. If the output DMA is running, stop it -----
        if (m_InputDMARunning)
        {
			pSSC->SSC_CR = SSC_CR_RXDIS;
            DMAD_StopTransfer(nDmaChannelRx);

            m_InputDMAStatus = DMA_CLEAR;
        }
    
        m_InputDMARunning = FALSE;
    
        CODEC_MuteControl(DMA_CH_IN, TRUE);
        CODEC_PowerControl();
        break;
        
    case WAPI_OUT:    
        //----- 1. If the output DMA is running, stop it -----
        if (m_OutputDMARunning)
        {
			DMAD_StopTransfer(nDmaChannelTx);
			m_OutputDMAStatus = DMA_CLEAR;
   			pSSC->SSC_CR = SSC_CR_TXDIS;


			//kprintf("%s(%d): DMA_STOP!!!!!!!\n", __FUNCTION__, __LINE__);
        }
    
        m_OutputDMARunning = FALSE;
    
        CODEC_MuteControl(DMA_CH_OUT, TRUE);
        CODEC_PowerControl();
        break;

    default:
        return FALSE;
    }
    
    return(TRUE);
}


STATIC VOID
DMA_UpdateBuffer( WAPI_INOUT wapidir, UINT32 nDMA, UINT32 pBuffer, UINT32 nLen)
{
    switch( wapidir )
    {
    case WAPI_IN:
//        while( (g_pDMAregs->DSTAT1&0x300000)!=0 && (g_pDMAregs->DSTAT1&0xfffff)==0 );
//        g_pDMAregs->DISRC1 = (UINT32)pBuffer;
		if(0 == nDMA)
		{
			DMA_RXBuffer = pBuffer + DMA_ADDR_OFFSET;
			DMA_RXBuf_Size = nLen;
			DMA_RXBuffer2 = 0;
			DMA_RXBuf2_Size = 0;
		}
		if(1 == nDMA)
		{
			DMA_RXBuffer2 = pBuffer + DMA_ADDR_OFFSET;
			DMA_RXBuf2_Size = nLen;
		}
        break;

    case WAPI_OUT:    
//        while( (g_pDMAregs->DSTAT2&0x300000)!=0 && (g_pDMAregs->DSTAT2&0xfffff)==0 );
//        g_pDMAregs->DISRC2 = (UINT32)pBuffer;
		
		if(0 == nDMA)
		{
			DMA_TXBuffer = pBuffer + DMA_ADDR_OFFSET;
			DMA_TXBuf_Size = nLen;
			//DMA_TXBuffer2 = 0;
			//DMA_TXBuf2_Size = 0;
		}
#if 1
		if(1 == nDMA)
		{
			DMA_TXBuffer2 = pBuffer + DMA_ADDR_OFFSET;
			DMA_TXBuf2_Size = nLen;
		}
#endif
//	DEBUGMSG( 1,"%s (%d): DMA_TXBuffer = %X, DMA_TXBuffer2 = %X\n", __FUNCTION__, __LINE__,DMA_TXBuffer,DMA_TXBuffer2 );
//	DEBUGMSG( 1,"%s (%d): DMA_TXBuf_Size = %d, DMA_TXBuf2_Size = %d\n", __FUNCTION__, __LINE__,DMA_TXBuf_Size,DMA_TXBuf2_Size );
		break;

    default:
        break;
    }
    
    return;
}




//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
// from here, HARDWARE INDEPENDENT CODE
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================
//==============================================================================



#define NUM_API_DIRS          2


typedef struct {
    UINT16    wSrcBitsPerSample;
    UINT16    wDstBitsPerSample;
    UINT16    wSrcNbChannels;
    UINT16    wDstNbChannels;
    BOOL        bInUse;        // is apidir in use? if so, fail Open calls
    UINT32      dwDMASize; // Size in bytes
} T_CHANNEL_DESC;


STATIC T_CHANNEL_DESC   gChannelDesc[NUM_API_DIRS];
STATIC BOOL                gbMoreData[NUM_API_DIRS];
STATIC UINT32           gnXmitComplete;             // number of byte already played by the codec

STATIC UINT32           gCaptureLatency;            // capture buffers will hold this much data (in milliseconds)

#define DEFAULT_CAPTURE_LATENCY         20



//==============================================================================
// buffer handle helper functions
//==============================================================================


typedef struct 
{
    PVOID   pData;
    UINT32  dwPhysAddr;
    UINT32  dwSize;
    UINT32  ID;
} T_BUFFER;

typedef struct 
{
    UINT32 dwListMaxSize;
    UINT32 dwNbBufferInList;
    UINT32 dwFirstBufferIndex;
    UINT32 dwNextBufferIndex;
    T_BUFFER *pBufferArray[MAX_BUFFER_PAGE];
} T_BUFFER_FIFO;


T_BUFFER_FIFO   FreeOutBufferFifo;          //! Fifo used to keep track of free emission buffer
T_BUFFER_FIFO   BufferToPlayFifo;           //! Fifo used to keep track of buffers ready to be played
T_BUFFER_FIFO   BufferBeingPlayedFifo;      //! Fifo used to keep track of buffers being played (already in the DMA list)
T_BUFFER        XmitBuffers[NB_XMIT_PAGE];

T_BUFFER_FIFO   FreeInBufferFifo;           //! Fifo used to keep track of buffer available for recording
T_BUFFER_FIFO   BufferRecordedFifo;         //! Fifo used to keep track of recorded buffers (after DMA)
T_BUFFER_FIFO   BufferBeingRecordedFifo;    //! Fifo used to keep track of buffers being recorded (already in the DMA list)
T_BUFFER        RcvBuffers[NB_RECV_PAGE];


STATIC VOID (*gpfnBufferFill) (PINT16 pDstBuffer, PVOID pSrcBuffer, UINT32 dwSamples);
STATIC VOID (*gpfnBufferGet)  (PVOID pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples);

#define min(a,b)                    (a>b?b:a)

//-----------------------------------------------------------------------------
//! \brief        This function creates a new Fifo
//! \param        dwNbBuffer the number of buffer that the fifo can handle
//! \param        pFifo       a pointer to the fifo structure that will be initialized
//-----------------------------------------------------------------------------
STATIC BOOL
BufferFifoInit( T_BUFFER_FIFO* pFifo )
{
    if (pFifo == NULL)
    {
        return FALSE;
    }
    
    pFifo->dwListMaxSize = NB_XMIT_BUFF_FIFO;

    pFifo->dwNbBufferInList = 0;
    pFifo->dwFirstBufferIndex = 0;
    pFifo->dwNextBufferIndex = 0;
    
    return TRUE;
}


//-----------------------------------------------------------------------------
//! \brief        This function resets a fifo
//! \param        pFifo       a pointer to the fifo structure that will be reset
//!
//! The fifo is emptied. There's no need to create the fifo again. It's still ready to use
//-----------------------------------------------------------------------------
STATIC BOOL
BufferFifoReset(T_BUFFER_FIFO* pFifo)
{
    if (pFifo == NULL)
    {
        return FALSE;
    }
    
    pFifo->dwNbBufferInList = 0;
    pFifo->dwFirstBufferIndex = 0;
    pFifo->dwNextBufferIndex = 0;
    
    return TRUE;
}


//-----------------------------------------------------------------------------
//! \brief        This function deletes a fifo
//! \param        pFifo       a pointer to the fifo structure that will be deleted
//!
//! The fifo is emptied. All allocations are released.
//-----------------------------------------------------------------------------
STATIC BOOL 
BufferFifoDelete( T_BUFFER_FIFO* pFifo )
{
    if (pFifo == NULL)
    {
        return FALSE;
    }
    
    pFifo->dwNbBufferInList = 0;
    pFifo->dwFirstBufferIndex = 0;
    pFifo->dwNextBufferIndex = 0;    
    pFifo->dwListMaxSize = 0;

    return TRUE;
}


//-----------------------------------------------------------------------------
//! \brief        This function gets the first buffer of the list and remove it from the fifo
//! \param        pFifo       a pointer to the fifo structure that will be deleted
//!
//! \return  A pointer to the first buffer of the fifo if it's not empty, else NULL
//-----------------------------------------------------------------------------
T_BUFFER* 
BufferFifoPop(T_BUFFER_FIFO* pFifo)
{
    T_BUFFER* pResult = NULL;
    
    if (pFifo == NULL)
    {
        return NULL;
    }

    if (pFifo->dwNbBufferInList != 0)
    {
        pResult = pFifo->pBufferArray[pFifo->dwFirstBufferIndex];        
        pFifo->dwFirstBufferIndex = INC_MODULO(pFifo->dwFirstBufferIndex,pFifo->dwListMaxSize);        
        pFifo->dwNbBufferInList--;
    }

    return pResult;
}

//-----------------------------------------------------------------------------
//! \brief        This function gets the first buffer of the list and remove it from the fifo
//! \param        pFifo       a pointer to the fifo structure that will be deleted
//! \param        pBuffer        a pointer to the buffer that must be stored in the fifo
//!
//! \return  TRUE indicates success, FALSE indicates failure
//-----------------------------------------------------------------------------
STATIC BOOL
BufferFifoPush( T_BUFFER_FIFO* pFifo,T_BUFFER* pBuffer )
{
    BOOL bResult = FALSE;
    if (pFifo == NULL)
    {
        return FALSE;
    }

    if (pFifo->dwNbBufferInList < pFifo->dwListMaxSize)
    {
        pFifo->pBufferArray[pFifo->dwNextBufferIndex] = pBuffer;
        pFifo->dwNextBufferIndex = INC_MODULO(pFifo->dwNextBufferIndex,pFifo->dwListMaxSize);        
        pFifo->dwNbBufferInList++;
        bResult = TRUE;
    }

    return bResult;
}



STATIC VOID
BufferFillM08toS16 (PINT16 pDstBuffer, PBYTE pSrcBuffer, UINT32 dwSamples)
{
    while (dwSamples-- > 0) 
    {
        PPCM_SAMPLE pSample = (PPCM_SAMPLE) pSrcBuffer;
        INT16 sample16 = BITS_8_TO_16(pSample->m8.sample);
        pDstBuffer[0] = sample16;
        pDstBuffer[1] = sample16;
        pDstBuffer += 2;
        pSrcBuffer += 1;
    }
}

STATIC VOID
BufferFillS08toS16 (PINT16 pDstBuffer, PBYTE pSrcBuffer, UINT32 dwSamples)
{
//kprintf("BufferFillS08toS16\n");
    while (dwSamples-- > 0) 
    {
        PPCM_SAMPLE pSample = (PPCM_SAMPLE) pSrcBuffer;
        pDstBuffer[0] = BITS_8_TO_16(pSample->s8.sample_left);
        pDstBuffer[1] = BITS_8_TO_16(pSample->s8.sample_right);
        pDstBuffer += 2;
        pSrcBuffer += 2;
    }
}

STATIC VOID
BufferFillM16toS16 (PINT16 pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples)
{
//kprintf("BufferFillM16toS16\n");
    while (dwSamples-- > 0)
    {
        INT16 sample = * (PINT16) pSrcBuffer;
        pDstBuffer[0] = sample;
        pDstBuffer[1] = sample;
        pDstBuffer += 2;
        pSrcBuffer += 1;

//kprintf("%x\n", sample);
    }
}

STATIC VOID
BufferFillS16toS16 (PINT16 pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples)
{
//kprintf("BufferFillS16toS16 : pDstBuffer = %x\n", pDstBuffer);
//kprintf("BufferFillS16toS16 : dwSamples = %x\n", dwSamples);
//kprintf("BufferFillS16toS16 : *pSrcBuffer = %x\n", *(pSrcBuffer+0);
//kprintf("BufferFillS16toS16 : *pSrcBuffer = %x\n", *(pSrcBuffer+2);
//kprintf("BufferFillS16toS16 : *pSrcBuffer = %x\n", *(pSrcBuffer+4);
#if 1
    memcpy(pDstBuffer, pSrcBuffer, dwSamples * sizeof(UINT32));
#elif 1

int j;
static int i = 0;
    for( j = 0  ; j < dwSamples ; j++ )
    {
        pDstBuffer[2*j+0] = (INT16)SineWave[i]/8;
        pDstBuffer[2*j+1] = (INT16)SineWave[i++]/8;
        
        if( i >= sizeof(SineWave)/2 ) //SineSize )
            i = 0;
    }
        
#elif 0

int j;
static INT16 i = 0;
    for( j = 0  ; j < dwSamples*2 ;  )
    {
        pDstBuffer[j++] = -j*8;
        pDstBuffer[j++] = -j*8;
        
        //if( i >= 0xFFF )
        //    i = 0;
    }
        
#else
short j;
for( j = 0; j < dwSamples ; j++ )
{
    if( j > dwSamples/2 )
    {
        pDstBuffer[2*j+0] = j;
        pDstBuffer[2*j+1] = j;
    }
    else
    {
        pDstBuffer[2*j+0] = (dwSamples/2 - j);
        pDstBuffer[2*j+1] = (dwSamples/2 - j);
    }
    
}
#endif
}


UINT32
BufferFill( PWAVEHDR pWaveHdr, PVOID pvDstBuffer, UINT32 dwBufferSize )
{
    PBYTE   pDstBuffer = (PBYTE)pvDstBuffer;
    PBYTE   pOldDstBuffer = pDstBuffer;
    UINT32  dwDstSamples = dwBufferSize / ((gChannelDesc[WAPI_OUT].wDstBitsPerSample/8)*gChannelDesc[WAPI_OUT].wDstNbChannels);

//DEBUGMSG( DBG_LVL_WAV,"%s(%d): gpfnBufferFill = %x\n",__FUNCTION__,__LINE__, gpfnBufferFill);

    while ((pWaveHdr != NULL) && (dwDstSamples > 0))
    {
        if (pWaveHdr->nByteRecorded >= pWaveHdr->BufferLength)
        {
            if (pWaveHdr->nByteRecorded > pWaveHdr->BufferLength)
            {
                //kprintf("SHOULD NEVER HAPPEN !!! pWaveHdr nByteRecorded %d buf length %d\n",pWaveHdr->nByteRecorded,pWaveHdr->BufferLength);
            }
            pWaveHdr = pWaveHdr->pNext;
        }
        else {
            UINT32 dwSrcSamples = (pWaveHdr->BufferLength - pWaveHdr->nByteRecorded) / ((gChannelDesc[WAPI_OUT].wSrcBitsPerSample/8)*gChannelDesc[WAPI_OUT].wSrcNbChannels);
            UINT32 dwSamples = min(dwDstSamples, dwSrcSamples);
            gpfnBufferFill((PINT16) pDstBuffer, pWaveHdr->pBuffer + pWaveHdr->nByteRecorded, dwSamples);
            pWaveHdr->nByteRecorded += dwSamples * ((gChannelDesc[WAPI_OUT].wSrcBitsPerSample/8)*gChannelDesc[WAPI_OUT].wSrcNbChannels);
//            g_TotalPlayed += dwSamples;
            pDstBuffer += dwSamples * ((gChannelDesc[WAPI_OUT].wDstBitsPerSample/8)*gChannelDesc[WAPI_OUT].wDstNbChannels);
            dwDstSamples -= dwSamples;
        }
    }

//kprintf("%s : (UINT32) pDstBuffer - (UINT32) pOldDstBuffer = %d\n", __FUNCTION__, (UINT32) pDstBuffer - (UINT32) pOldDstBuffer);
    return (UINT32) pDstBuffer - (UINT32) pOldDstBuffer;
}

VOID
BufferGetM08fromM16 (PBYTE pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples)
{
    while (dwSamples-- > 0)
    {
        *pDstBuffer = (BYTE) BITS_16_TO_8(pSrcBuffer[0]);
        pSrcBuffer += 1;
        pDstBuffer += 1;
    }
}

VOID
BufferGetS08fromM16 (PBYTE pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples)
{
    register INT8 i8Value;
    while (dwSamples-- > 0)
    {
        i8Value = (BYTE) BITS_16_TO_8(pSrcBuffer[0]);
        pDstBuffer[0] = i8Value;
        pDstBuffer[1] = i8Value;
        pSrcBuffer += 1;
        pDstBuffer += 2;
    }
}


VOID
BufferGetM16fromM16 (PINT16 pDst, PINT16 pSrcBuffer, UINT32 dwSamples)
{
    memcpy(pDst, pSrcBuffer, dwSamples * sizeof(INT16));
}


VOID
BufferGetS16fromM16 (PINT16 pDstBuffer, PINT16 pSrcBuffer, UINT32 dwSamples)
{
    register INT16 i16Value;
    while (dwSamples-- > 0) 
    {
        i16Value = pSrcBuffer[0];
        pDstBuffer[0] = i16Value;
        pDstBuffer[1] = i16Value;
        pSrcBuffer += 1;
        pDstBuffer += 2;
    }
}

UINT32
BufferGet( PWAVEHDR pWaveHdr, PVOID pvSrcBuffer, UINT32 dwBufferSize )
{
    PBYTE pSrcBuffer = pvSrcBuffer;
    PVOID pOldSrcBuffer = pSrcBuffer;
    UINT32 dwSrcSamples = dwBufferSize / ((gChannelDesc[WAPI_IN].wSrcBitsPerSample/8)*gChannelDesc[WAPI_IN].wSrcNbChannels);
//kprintf("+BufferGet\n");
    
    while ((pWaveHdr != NULL) && (dwSrcSamples > 0))
    {
        if (pWaveHdr->nByteRecorded >= pWaveHdr->BufferLength)
        {
            pWaveHdr = pWaveHdr->pNext;
        }
        else
        {
            UINT32 dwDstSamples = (pWaveHdr->BufferLength - pWaveHdr->nByteRecorded) / ((gChannelDesc[WAPI_IN].wDstBitsPerSample/8)*gChannelDesc[WAPI_IN].wDstNbChannels);
            UINT32 dwSamples = min(dwSrcSamples, dwDstSamples);
            gpfnBufferGet(pWaveHdr->pBuffer + pWaveHdr->nByteRecorded, (PINT16)pSrcBuffer, dwSamples);
            pWaveHdr->nByteRecorded += dwSamples * ((gChannelDesc[WAPI_IN].wDstBitsPerSample/8)*gChannelDesc[WAPI_IN].wDstNbChannels);
            pSrcBuffer += dwSamples * ((gChannelDesc[WAPI_IN].wSrcBitsPerSample/8)*gChannelDesc[WAPI_IN].wSrcNbChannels);
            dwSrcSamples -= dwSamples;
        }
    }
    
    return (UINT32) pSrcBuffer - (UINT32) pOldSrcBuffer;
}




//==============================================================================
// interface functions
//==============================================================================

PUBLIC BOOL
WavHal_Init( UINT32 Minor )
{
    int i;

	DEBUGMSG(DGB_LVL_IF,"%s(%d): Minor = %x\n",__FUNCTION__,__LINE__, Minor);

    //Initialize structure
    gChannelDesc[WAPI_OUT].bInUse = FALSE;
    gChannelDesc[WAPI_IN].bInUse = FALSE;


//    //
//    // Configure the PIOs
//    //
    const struct pio_desc hw_pio[] = {
		{"TWD",       AT91C_PIN_PA(30),   0, PIO_DEFAULT, PIO_PERIPH_A},
		{"TWCK",      AT91C_PIN_PA(31),   0, PIO_DEFAULT, PIO_PERIPH_A},
		{"SSCTD",     AT91C_PIN_PA(26),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"SSCTK",     AT91C_PIN_PA(24),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"SSCTF",     AT91C_PIN_PA(25),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"SSCRD",     AT91C_PIN_PA(27),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"SSCRK",     AT91C_PIN_PA(28),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"SSCTD",     AT91C_PIN_PA(29),   0, PIO_DEFAULT, PIO_PERIPH_B},
		{"MCLK",	  AT91C_PIN_PC(31),   0, PIO_DEFAULT, PIO_PERIPH_C},
	};

    pio_setup(hw_pio, sizeof(hw_pio)/sizeof(struct pio_desc));

    // Map both DMA pages into the local address space.
    PBYTE AudioDmaPage = (PBYTE)(PBYTE)MemPageAlloc(4);

    OutputDMAPage[OUTPUT_DMA_PAGE0]   = (PBYTE)(AudioDmaPage + AUDIO_DMA_PAGE_SIZE*0);
    OutputDMAPage[OUTPUT_DMA_PAGE1]   = (PBYTE)(AudioDmaPage + AUDIO_DMA_PAGE_SIZE*1);
    InputDMAPage[INPUT_DMA_PAGE0]     = (PBYTE)(AudioDmaPage + AUDIO_DMA_PAGE_SIZE*2);
    InputDMAPage[INPUT_DMA_PAGE1]     = (PBYTE)(AudioDmaPage + AUDIO_DMA_PAGE_SIZE*3);

    memset( AudioDmaPage, 0x0F, AUDIO_DMA_PAGE_SIZE*2 );

    // Codec Power On
    CODEC_PowerOn();

    //Set the default values first
    gCaptureLatency = DEFAULT_CAPTURE_LATENCY;

	SSC_Init(SAMPLE_RATE * BITS_BY_SLOT * 2, BOARD_MCK);
	Sleep(100);
    CODEC_Init();

    //Setup the buffers 
    for( i=0; i<NB_XMIT_PAGE; i++ )
    {
        XmitBuffers[i].pData = OutputDMAPage[i];
        XmitBuffers[i].dwPhysAddr = ((UINT32)OutputDMAPage[i]);
        XmitBuffers[i].ID = i;
    }

    for( i=0; i<NB_RECV_PAGE; i++ )
    {
        RcvBuffers[i].pData = InputDMAPage[i] ;
        RcvBuffers[i].dwPhysAddr = ((UINT32)InputDMAPage[i]);
        RcvBuffers[i].ID = i;
    }

    // Create the fifos
    BufferFifoInit(&FreeOutBufferFifo);
    BufferFifoInit(&BufferToPlayFifo);
    BufferFifoInit(&BufferBeingPlayedFifo);
    
    BufferFifoInit(&FreeInBufferFifo);
    BufferFifoInit(&BufferRecordedFifo);
    BufferFifoInit(&BufferBeingRecordedFifo);
        

    DMA_Init( WAPI_OUT );
	Sleep(100);
    DMA_Init( WAPI_IN );


    WAV_DeviceConfig.nInterrupt = snSysIntr;

    return TRUE;
}




PUBLIC AUDIO_STATE
WavHal_GetInterruptType( UINT32 Param )
{
    AUDIO_STATE RetStatus;
    BOOL        dwBothBufferConsummed;
    UINT32      dmaInterruptSource;
    UINT32      IntStatus;
	AT91PS_SSC pSSC = (AT91PS_SSC)AT91C_BASE_SSC0;
	
    RetStatus = AUDIO_STATE_IGNORE ;


    dmaInterruptSource = 0;


    IntStatus = Param;

    if( IntStatus & (1 << DMA_CH_IN) )
    {
        dmaInterruptSource |= DMA_CH_MIC;                                // Input DMA is supported...
    }
    
    if( IntStatus & (1 << DMA_CH_OUT) )
    {
        dmaInterruptSource |= DMA_CH_OUT;                                // Output DMA is supported...
    }

	DEBUGMSG( DBG_LVL_WAV,"%s(%d): IntStatus = %x\n",__FUNCTION__,__LINE__, IntStatus);
	DEBUGMSG( DBG_LVL_WAV,"%s(%d): dmaInterruptSource = %x\n",__FUNCTION__,__LINE__, dmaInterruptSource);
	DEBUGMSG( DBG_LVL_WAV,"%s(%d): gbMoreData[WAPI_IN] = %x\n",__FUNCTION__,__LINE__, gbMoreData[WAPI_IN]);
	DEBUGMSG( DBG_LVL_WAV,"%s(%d): gbMoreData[WAPI_OUT] = %x\n",__FUNCTION__,__LINE__, gbMoreData[WAPI_OUT]);

	

    // for input (MIC-IN)
if( (dmaInterruptSource & DMA_CH_MIC) )
{
	DMAD_Handler(nDmaChannelRx);

    if( (gbMoreData[WAPI_IN] == TRUE) )
    {

        if(m_InputDMAStatus & DMA_BIU)
        {
            if( m_InputDMAStatus & DMA_STRTB )
            {
                // pop the buffer from beingplayed fifo
                T_BUFFER * pBuffer;
                pBuffer = BufferFifoPop(&BufferBeingRecordedFifo);
                if (pBuffer)
                {
                    BufferFifoPush(&BufferRecordedFifo,pBuffer);
                }
            }

            m_InputDMAStatus &= ~DMA_STRTB;                        // Buffer A just completed...
            m_InputDMAStatus |= DMA_DONEB;

            m_InputDMAStatus &= ~DMA_BIU;                            // Buffer B is in use
            
            //while((g_pDMAregs->DSTAT2&0xfffff)==0);
            //SELECT_AUDIO_DMA_OUTPUT_BUFFER_B();        // charlie. B => A
        }
        else
        {
            if( m_InputDMAStatus & DMA_STRTA )
            {
                // pop the buffer from beingplayed fifo
                T_BUFFER * pBuffer;
                pBuffer = BufferFifoPop(&BufferBeingRecordedFifo);
                if (pBuffer)
                {
                    BufferFifoPush(&BufferRecordedFifo,pBuffer);
                }
            }

            m_InputDMAStatus &= ~DMA_STRTA;                        // Buffer B just completed...
            m_InputDMAStatus |= DMA_DONEA;

            m_InputDMAStatus |= DMA_BIU;                            // Buffer A is in use
            
            //while((g_pDMAregs->DSTAT2&0xfffff)==0);
            //SELECT_AUDIO_DMA_OUTPUT_BUFFER_A();        // charlie. B => A
        }



        dwBothBufferConsummed = TRUE;

        switch(m_InputDMAStatus&(DMA_DONEA|DMA_DONEB|DMA_BIU))
        {
        case 0:
        case DMA_BIU:
            // No done bits set- must not be my interrupt
            RetStatus |= AUDIO_STATE_IGNORE;
            break;;
        case DMA_DONEA|DMA_DONEB|DMA_BIU:
        case DMA_DONEA|DMA_DONEB:
            {
                // for safty....
                // pop the all buffer from beingplayed fifo
                T_BUFFER * pBuffer;
                pBuffer = BufferFifoPop(&BufferBeingRecordedFifo);
                if (pBuffer)
                {
                    BufferFifoPush(&BufferRecordedFifo,pBuffer);
                }
            }

            RetStatus |= AUDIO_STATE_IN_OVERFLOW;
            break;
        case DMA_DONEA: // This should never happen!
        case DMA_DONEA|DMA_BIU:
        case DMA_DONEB|DMA_BIU: // This should never happen!
        case DMA_DONEB:
            dwBothBufferConsummed = FALSE;
            RetStatus |= AUDIO_STATE_IN_RECORDING;
            break;
        }

        if (dwBothBufferConsummed)
        {
            T_BUFFER *pBuffer;
            T_BUFFER *pBuffer2;
            pBuffer = BufferFifoPop(&FreeInBufferFifo);                
            pBuffer2 = BufferFifoPop(&FreeInBufferFifo);
            
            if (pBuffer)
            {
                DMA_UpdateBuffer(WAPI_IN, 0, pBuffer->dwPhysAddr, pBuffer->dwSize );
                
                BufferFifoPush(&BufferBeingRecordedFifo,pBuffer);
                
                if (pBuffer2)
                {
//kprintf("recording %d puis %d\n",pBuffer->ID,pBuffer2->ID);
                    DMA_UpdateBuffer(WAPI_IN, 1, pBuffer2->dwPhysAddr, pBuffer2->dwSize );
                    BufferFifoPush(&BufferBeingRecordedFifo,pBuffer2);
                }
                else
                {
//kprintf("recording %d\n",pBuffer->ID);
                }
                
                //Enable the Rx interrupt
                DMA_Start(WAPI_IN);
                
                RetStatus |= AUDIO_STATE_IN_RECORDING;
            }    
            else
            {
//kprintf("int handler. No buffer to record in the Fifo\n");
                //disable PDC TX interrupt 
                DMA_Stop(WAPI_IN);

                RetStatus |= AUDIO_STATE_IN_RECORDING;          //TODO: Does the MDD now send a WPDM_STOP message?
            }
        }
        else
        {
            T_BUFFER *pBuffer;
            pBuffer = BufferFifoPop(&FreeInBufferFifo);

            if (pBuffer)
            {
//kprintf("recording seulement %d \n",pBuffer->ID);
                DMA_UpdateBuffer(WAPI_IN, 0, pBuffer->dwPhysAddr, pBuffer->dwSize );

                //Enable the Tx interrupt
                DMA_Start(WAPI_IN);
                
                BufferFifoPush(&BufferBeingRecordedFifo,pBuffer);
                
                RetStatus |= AUDIO_STATE_IN_RECORDING;
            }    
            else
            {
//kprintf("int handler. No buffer to record in the Fifo\n");
                //disable PDC TX interrupt 
                DMA_Stop(WAPI_IN);
                RetStatus |= AUDIO_STATE_IN_RECORDING;          //TODO: Does the MDD now send a WPDM_STOP message?
            }
        }

    }
    else
    {
        //disable PDC RX interrupt
        DMA_Stop(WAPI_IN);
        
        RetStatus |= AUDIO_STATE_IN_STOPPED;           //JJH TODO: Does the MDD now send a WPDM_STOP message?
    }
}

    // for output, (SOUNT-OUT )
if( (dmaInterruptSource & DMA_CH_OUT) )
{
	UINT32	nRet;
	UINT32	DMA_Status;
    BYTE _iController = (nDmaChannelTx >> 3);
    BYTE iChannel     = nDmaChannelTx & 0x7; //(nChannel) & 0xFF;
	PSHARED_MEMORY  pSharedMemory = (PSHARED_MEMORY)SHARED_MEMORY_BASE;

	DMA_Status = pSharedMemory->DMAC_reg_EBCISR[_iController] & (0x010101 << iChannel);

	DEBUGMSG(0,"%s(%d): DMA_Status = %x\n", __FUNCTION__, __LINE__, DMA_Status);
	DMAD_Handler(nDmaChannelTx);
	
    if( (gbMoreData[WAPI_OUT] == TRUE ) )
    {
		DEBUGMSG(0,"%s(%d): RetStatus = %x\n", __FUNCTION__, __LINE__, RetStatus);

        T_BUFFER * pBuffer;
        pBuffer = BufferFifoPop(&BufferBeingPlayedFifo);
		DEBUGMSG(0,"%s(%d): pBuffer = %x\n", __FUNCTION__, __LINE__, pBuffer);
        if (pBuffer)
        {
            gnXmitComplete += pBuffer->dwSize;
            pBuffer->dwSize = gChannelDesc[WAPI_OUT].dwDMASize;
            BufferFifoPush(&FreeOutBufferFifo,pBuffer);
        }


        RetStatus |= AUDIO_STATE_OUT_PLAYING;
    }
    else
    {
        //disable PDC TX interrupt 
		//kprintf("%s(%d): DMA_STOP\n",__FUNCTION__, __LINE__);
        DMA_Stop(WAPI_OUT);
        
        RetStatus |= AUDIO_STATE_OUT_STOPPED;          //TODO: Does the MDD now send a WPDM_STOP message?
//DEBUGMSG(DGB_LVL_IF," WavHal_GetInterruptType : RetStatus8 = %x\n", RetStatus);
    }
}
//DEBUGMSG(DGB_LVL_IF," WavHal_GetInterruptType = %x\n", RetStatus);

    return RetStatus;
}




PUBLIC BOOL
WavHal_WaveOpen( WAPI_INOUT apidir, PWAVEFORMAT pFormat )
{

DEBUGMSG(DGB_LVL_IF," pFormat = %X\n", pFormat);

DEBUGMSG(DGB_LVL_IF," FormatTag = %d\n", pFormat->FormatTag);
DEBUGMSG(DGB_LVL_IF," nChannel = %d\n", pFormat->nChannel);
DEBUGMSG(DGB_LVL_IF," nSamplesPerSec = %d\n", pFormat->nSamplesPerSec);
DEBUGMSG(DGB_LVL_IF," BitsPerSample = %d\n", pFormat->BitsPerSample);

    //invalid direction : no input audio source
    if( apidir >= NUM_API_DIRS )
    {
        DEBUG1(DGB_LVL_IF,"Invalid Direction\r\n ");
        return FALSE;
    }

    if( pFormat->FormatTag != WAVE_FORMAT_PCM )
    {
        DEBUG1(DGB_LVL_IF, "Invalid bad format\n");
        return FALSE;
    }
        
    //invalid bit per sample
    if( pFormat->BitsPerSample != 8 && pFormat->BitsPerSample != 16)
    {
        DEBUG1(DGB_LVL_IF,"Invalid bits per sample\r\n ");
        return FALSE;
    }

    //invalid bit nbChannels
    if( pFormat->nChannel !=1  && pFormat->nChannel != 2)
    {
        DEBUG1(DGB_LVL_IF,"Invalid number of channels\r\n ");
        return FALSE;
    }

    //invalid bit rate 
    if( pFormat->nSamplesPerSec != KHZ08_000 &&
        pFormat->nSamplesPerSec != KHZ11_025 &&
        pFormat->nSamplesPerSec != KHZ16_000 &&
        pFormat->nSamplesPerSec != KHZ22_050 &&
        pFormat->nSamplesPerSec != KHZ32_000 &&
        pFormat->nSamplesPerSec != KHZ44_100 &&
        pFormat->nSamplesPerSec != KHZ48_000 )
    {
        DEBUG1(DGB_LVL_IF,"Invalid samples per second\r\n ");
        return FALSE;
    }

    //channel already in use
    if( gChannelDesc[apidir].bInUse )
    {
        DEBUG1(DGB_LVL_IF, "channel already in use\r\n");
        return FALSE;
    }
    
    gChannelDesc[apidir].bInUse = TRUE;

    if (apidir == WAPI_OUT)
    {
        gChannelDesc[apidir].wSrcBitsPerSample = pFormat->BitsPerSample;
        gChannelDesc[apidir].wDstBitsPerSample = 16;
        gChannelDesc[apidir].wSrcNbChannels = pFormat->nChannel;
        gChannelDesc[apidir].wDstNbChannels = 2;
    }
    else
    {
        gChannelDesc[apidir].wSrcBitsPerSample = pFormat->BitsPerSample;
        gChannelDesc[apidir].wDstBitsPerSample = 16;
        gChannelDesc[apidir].wSrcNbChannels = pFormat->nChannel;
        gChannelDesc[apidir].wDstNbChannels = 1;
    }

    if( apidir == WAPI_OUT )
    {
        if (pFormat->BitsPerSample == 8)
        {
            if (pFormat->nChannel == 1)
                gpfnBufferFill = BufferFillM08toS16;
            else
                gpfnBufferFill = BufferFillS08toS16;
        }
        else
        {
            if (pFormat->nChannel == 1)
                gpfnBufferFill = BufferFillM16toS16;
            else
                gpfnBufferFill = BufferFillS16toS16;
        }
        gChannelDesc[apidir].dwDMASize = AUDIO_DMA_PAGE_SIZE;
    }
    else
    {
        if( pFormat->BitsPerSample == 8 )
        {
            if (pFormat->nChannel == 1)
                gpfnBufferGet = BufferGetM08fromM16;
            else
                gpfnBufferGet = BufferGetS08fromM16;
        }
        else
        {
            if (pFormat->nChannel == 1)
                gpfnBufferGet = BufferGetM16fromM16;
            else
                gpfnBufferGet = BufferGetS16fromM16;
        }

        // Support Low-latency Capture
        // Adjust the effective DMA buffer size, based on the requested sample rate.
        // The idea is that we want our DMA capture buffers to represent a fixed amount of time,
        // regardless of the sample rate.
        // This means that if the application buffers are small (as is the case with VoIP or DirectSoundCapture)
        // then we still can return the buffer very soon after we have enough captured data to fill it.
        // The way this used to work (fixed-size capture buffers) meant that at low sample rates
        // we would wait for hundreds of milliseconds, then turn around and discover that the application
        // had only given us 80 milliseconds of wave headers, so we would be forced to drop samples on the floor.

        gChannelDesc[WAPI_IN].dwDMASize = ((UINT32) gChannelDesc[apidir].wSrcBitsPerSample / 8 * (UINT32) gChannelDesc[apidir].wSrcNbChannels * (UINT32)pFormat->nSamplesPerSec * (UINT32) gCaptureLatency /1000);
        if (gChannelDesc[WAPI_IN].dwDMASize > AUDIO_DMA_PAGE_SIZE) 
            gChannelDesc[WAPI_IN].dwDMASize = AUDIO_DMA_PAGE_SIZE;
    }

   
    //set the sample rate
    if (AudioSetSampleRate(apidir, (UINT16)pFormat->nSamplesPerSec) == FALSE)
    {
DEBUG1(DGB_LVL_IF,"return bad sample rate:\n");
        return FALSE;
    }   


    //Enable DAC or ADC
    if(apidir == WAPI_OUT)
    {
        AudioPowerDownUnit(WAPI_OUT,FALSE);
        WaveOutMute(FALSE);
    }
    else
    {
        WaveInMute(FALSE);   
        AudioPowerDownUnit(WAPI_IN,FALSE);
        Sleep(100);     //Give the Codec some time to settle
    }

    return TRUE;
}


PUBLIC VOID
WavHal_WaveOutStart( PWAVEHDR pWaveHdr )
{
    T_BUFFER* pBuffer;
    T_BUFFER* pBuffer2;
    UINT32 i;

DEBUG1(0,"++WavHal_WaveOutStart\n");
    

    // Reinit our fifos 
    BufferFifoReset(&FreeOutBufferFifo);
    BufferFifoReset(&BufferToPlayFifo);
    BufferFifoReset(&BufferBeingPlayedFifo);    
    for (i=0;i<NB_XMIT_PAGE;i++)
    {
        XmitBuffers[i].dwSize = gChannelDesc[WAPI_OUT].dwDMASize;
        BufferFifoPush(&FreeOutBufferFifo,&XmitBuffers[i]);
    }

    gbMoreData[WAPI_OUT] = TRUE;        // we expect to have more data coming...

    pBuffer = BufferFifoPop(&FreeOutBufferFifo);
    if (pBuffer)
    {
        MemSet( pBuffer->pData, 0 , gChannelDesc[WAPI_OUT].dwDMASize );
        pBuffer->dwSize = gChannelDesc[WAPI_OUT].dwDMASize ; 
        BufferFifoPush(&BufferToPlayFifo,pBuffer);
    }
    pBuffer = BufferFifoPop(&FreeOutBufferFifo);
    if (pBuffer)
    {
        MemSet( pBuffer->pData, 0 , gChannelDesc[WAPI_OUT].dwDMASize );
        pBuffer->dwSize = gChannelDesc[WAPI_OUT].dwDMASize ; 
        BufferFifoPush(&BufferToPlayFifo,pBuffer);
    }

    pBuffer = BufferFifoPop(&BufferToPlayFifo);
    pBuffer2 = BufferFifoPop(&BufferToPlayFifo);

    DMA_UpdateBuffer( WAPI_OUT, 0, pBuffer->dwPhysAddr, pBuffer->dwSize );
    BufferFifoPush(&BufferBeingPlayedFifo,pBuffer);

    DMA_UpdateBuffer( WAPI_OUT, 1, pBuffer2->dwPhysAddr, pBuffer2->dwSize );
    BufferFifoPush(&BufferBeingPlayedFifo,pBuffer2);

	
    DMA_Start(WAPI_OUT);

DEBUG1(DGB_LVL_IF,"--WavHal_WaveOutStart\n");
    return;
}



PUBLIC VOID
WavHal_WaveOutEndOfData( VOID )
{
DEBUG1(0,"+WavHal_WaveOutEndOfData\n");

    //
    // There is no more data coming...
    //
    gbMoreData[WAPI_OUT] = FALSE;

//kprintf("%s(%d): DMA_STOP\n",__FUNCTION__, __LINE__);
    DMA_Stop(WAPI_OUT);

    return;
}

PUBLIC VOID
WavHal_WaveOutStop( VOID )
{
DEBUG1(0,"+WavHal_WaveOutStop");

    //WavHal_WaveOutEndOfData();
    gbMoreData[WAPI_OUT] = FALSE;

//kprintf("%s(%d): DMA_STOP\n",__FUNCTION__, __LINE__);
    DMA_Stop(WAPI_OUT);

    return;
}


PUBLIC VOID
WavHal_WaveOutStandby( VOID )
{
DEBUG1(DGB_LVL_IF,"+WavHal_WaveOutStandby\n");

    // We are going to be idle. so power off what we can.

//    AudioPowerDownUnit(WAPI_OUT,TRUE);  //turn

    //AudioPowerDownUnit(PWR_PR0_ADC,PowerDownIn);  //turn
        
}

PUBLIC BOOL
WavHal_WaveOutPause ( VOID )
{
DEBUG1(DGB_LVL_IF,"+WavHal_WaveOutPause\n");

    WavHal_WaveOutEndOfData();

    return TRUE;
}

PUBLIC BOOL
WavHal_WaveOutRestart( PWAVEHDR pWaveHdr )
{
DEBUG1(DGB_LVL_IF,"+WavHal_WaveOutRestart\n");

    WavHal_WaveOutStart (pWaveHdr);

    return TRUE;
}



PUBLIC UINT32
WavHal_WaveOutContinue( PWAVEHDR pWaveHdr )
{
    T_BUFFER * pBuffer;
    BOOL bLoop = TRUE;
    
DEBUG1(DGB_LVL_IF,"+WavHal_WaveOutContinue\n");

    gnXmitComplete = 0;

    do
    {        
        pBuffer = BufferFifoPop(&FreeOutBufferFifo);
		//DEBUGMSG(1,"%s(%d): pBuffer = %x\n",__FUNCTION__, __LINE__, pBuffer);
        if (pBuffer)
        {
            UINT32 dwSize;                        
            
            dwSize = BufferFill(pWaveHdr, (PUINT32) pBuffer->pData, pBuffer->dwSize);
			DEBUGMSG(DGB_LVL_IF,"%s(%d): ID %d : %d / %d\r\n",__FUNCTION__, __LINE__, pBuffer->ID, pBuffer->dwSize,dwSize);

            if (pBuffer->dwSize != dwSize) //Is the buffer entirely filled ?            
            {                
                //The buffer hasn't filled entirely, then we have no more data available
                bLoop = FALSE;
            }
            
                
            if (dwSize)
            {
                pBuffer->dwSize = dwSize;
                BufferFifoPush(&BufferToPlayFifo,pBuffer);
            }
            else
            {
                BufferFifoPush(&FreeOutBufferFifo,pBuffer);
            }
        }
    } while (bLoop && pBuffer);


    {
        T_BUFFER *pBuffer2;

        pBuffer = BufferFifoPop(&BufferToPlayFifo);                
        pBuffer2 = BufferFifoPop(&BufferToPlayFifo);
DEBUGMSG( 0, "%s : pBuffer  = %x\n", __FUNCTION__, pBuffer );
DEBUGMSG( 0, "%s : pBuffer2 = %x\n", __FUNCTION__, pBuffer2 );
        if( pBuffer && pBuffer2 )
        {
            // enable the fifo to the DMA
            DMA_UpdateBuffer(WAPI_OUT, 0, pBuffer->dwPhysAddr, pBuffer->dwSize );
            BufferFifoPush(&BufferBeingPlayedFifo,pBuffer);

            DMA_UpdateBuffer(WAPI_OUT, 1, pBuffer2->dwPhysAddr, pBuffer2->dwSize );
            BufferFifoPush(&BufferBeingPlayedFifo,pBuffer2);
        }
        else
        {
            if (pBuffer)
            {
//kprintf("playing %d\n",pBuffer->ID);
				DMA_UpdateBuffer(WAPI_OUT, pBuffer->ID, pBuffer->dwPhysAddr, pBuffer->dwSize );
                
                //Enable the Tx interrupt
                //DMA_Start(WAPI_OUT);

                BufferFifoPush(&BufferBeingPlayedFifo,pBuffer);
            }    
            else
            {
//kprintf("int handler. No buffer to play in the Fifo\n");
                //disable PDC TX interrupt 
//kprintf("%s(%d): DMA_STOP\n",__FUNCTION__, __LINE__);
                DMA_Stop(WAPI_OUT);
            }
        }        
	}
    
    //Enable the PDC tx interrupt .. just in case it had been disabled (no more data available)
    //DMA_Start( WAPI_OUT );


    return 0;
}


PUBLIC UINT32
WavHal_WaveOutClose( VOID )
{
DEBUG1(1,"+WavHal_WaveOutClose\n");

    //IIS_Deinitialize();

    return 0;    
}





//------------------------------------------------------------------------------------------------------------
//! \brief the WPDM_START message where the WAPI_INOUT direction is WAPI_IN.
//-------------------------------------------------------------------------------------------------------------
PUBLIC VOID
WavHal_WaveInStart( PWAVEHDR pWaveHdr )
{        
    T_BUFFER* pBuffer;
    T_BUFFER* pBuffer2;
    UINT32 i;

DEBUG1(DGB_LVL_IF,"+WavHal_WaveInStart\n");

    BufferFifoReset(&FreeInBufferFifo);
    BufferFifoReset(&BufferRecordedFifo);
    BufferFifoReset(&BufferBeingRecordedFifo);

    for (i=0;i<NB_XMIT_PAGE;i++)
    {
        RcvBuffers[i].dwSize = gChannelDesc[WAPI_IN].dwDMASize;
        BufferFifoPush(&FreeInBufferFifo,&RcvBuffers[i]);
    }

    //clear the mute
    WaveInMute(FALSE);     

    gbMoreData[WAPI_IN]  = TRUE;        // we expect to have more data coming...

    pBuffer = BufferFifoPop(&FreeInBufferFifo);
    pBuffer2 = BufferFifoPop(&FreeInBufferFifo);
    
    if (pBuffer && pBuffer2)
    {

        DMA_UpdateBuffer( WAPI_IN, 0, pBuffer->dwPhysAddr, pBuffer->dwSize );
        DMA_UpdateBuffer( WAPI_IN, 1, pBuffer2->dwPhysAddr, pBuffer2->dwSize );

        //DMA_Start( WAPI_IN );
        //g_pAC97C->AC97C_PTCR = AT91C_PDC_RXTEN;        

        BufferFifoPush(&BufferBeingRecordedFifo,pBuffer);
        BufferFifoPush(&BufferBeingRecordedFifo,pBuffer2);
    }        

    //Enable the Rx interrupt
    DMA_Start( WAPI_IN );
    //g_pAC97C->AC97C_CAMR |= AT91C_AC97C_ENDRX | AT91C_AC97C_RXBUFF | AT91C_AC97C_OVRUN;

    return;    
}

//------------------------------------------------------------------------------------------------------------
//! \brief  instruct the audio PDD to continue with the data pointed to by the wave header
//-------------------------------------------------------------------------------------------------------------
PUBLIC VOID
WavHal_WaveInContinue( PWAVEHDR pWaveHdr )
{    
    T_BUFFER * pBuffer;
    UINT32 dwSize;
    BOOL bLoop = TRUE;
    
DEBUG1(DGB_LVL_IF, "+WavHal_WaveInContinue\n");
    
    do
    {
        pBuffer = BufferFifoPop(&BufferRecordedFifo);
        if (pBuffer)
        {            
//kprintf("getting buffer %d\n",pBuffer->ID);
            dwSize = BufferGet(pWaveHdr, pBuffer->pData, pBuffer->dwSize);
            
            BufferFifoPush(&FreeInBufferFifo,pBuffer);            
        }
        bLoop = FALSE;
    } while (pBuffer && bLoop);
    
    //Enable the Rx interrupt
    DMA_Start( WAPI_IN );
    //g_pAC97C->AC97C_CAMR |= AT91C_AC97C_ENDRX | AT91C_AC97C_RXBUFF | AT91C_AC97C_OVRUN;
    
    return;
}


PUBLIC VOID
WavHal_WaveInStandby( VOID )
{
DEBUG1(DGB_LVL_IF,"+WavHal_WaveInStandby\n");

    // We are going to be idle. so power off what we can.

    AudioPowerDownUnit(WAPI_IN, TRUE);  //turn
        
}

//------------------------------------------------------------------------------------------------------------
//! \brief stop recording immediately
//-------------------------------------------------------------------------------------------------------------
PUBLIC VOID
WavHal_WaveInStop(PWAVEHDR pWaveHdr )
{
    T_BUFFER * pBuffer;

DEBUG1(DGB_LVL_IF,"+WavHal_WaveInStop\n");
    
    gbMoreData[WAPI_IN] = FALSE;
    
    DMA_Stop(WAPI_IN);
    
    WaveInMute(TRUE);      
    
    do
    {
        pBuffer = BufferFifoPop(&BufferRecordedFifo);
        if (pBuffer)
        {
            BufferGet(pWaveHdr, pBuffer->pData, pBuffer->dwSize);
            BufferFifoPush(&FreeInBufferFifo,pBuffer);
        }
    } while (pBuffer);

    return;    
}

STATIC BOOL
WaveOutSetPath( UINT32 SetPath )
{
    USHORT Left, Right;
	
	Right	= (USHORT) ((SetPath & 0x0000ffff) ); 
	Left	= (USHORT) (((SetPath & 0xffff0000)>>16) );

#if 0   // always on
		CODEC_WriteRegister( WM8960_ROUTMIX2, m_WM8580_SFR_Table[WM8960_ROUTMIX2] | (1 << 8) );//RD2RO Enable Path
#else
	if( Right & (1<<0))
	{
		CODEC_WriteRegister( WM8960_ROUTMIX2, m_WM8580_SFR_Table[WM8960_ROUTMIX2] | (1 << 8) );//RD2RO Enable Path
	}
	else
	{
		CODEC_WriteRegister( WM8960_ROUTMIX2, m_WM8580_SFR_Table[WM8960_ROUTMIX2] & ~(1 << 8) );//RD2RO Disable Path
	}
#endif

	if( Right & (1<<1))
	{
		CODEC_WriteRegister( WM8960_BYPASS2, m_WM8580_SFR_Table[WM8960_BYPASS2] | (1 << 7) );//RB2RO Enable Path
	}
	else
	{
		CODEC_WriteRegister( WM8960_BYPASS2, m_WM8580_SFR_Table[WM8960_BYPASS2] & ~(1 << 7) );//RB2RO Disable Path
	}
	
	
	
	if(Left & (1<<0))
	{
		CODEC_WriteRegister( WM8960_LOUTMIX1, m_WM8580_SFR_Table[WM8960_LOUTMIX1] | (1 << 8) );	//LD2LO Enable
	}
	else
	{
		CODEC_WriteRegister( WM8960_LOUTMIX1, m_WM8580_SFR_Table[WM8960_LOUTMIX1] & ~(1 << 8));	//LD2LO Disable
		
	}

	if(Left & (1<<1))
	{
		CODEC_WriteRegister( WM8960_BYPASS1, m_WM8580_SFR_Table[WM8960_BYPASS1] | (1 << 7) );	//LB2LO Enable
	}
	else
	{
		CODEC_WriteRegister( WM8960_BYPASS1, m_WM8580_SFR_Table[WM8960_BYPASS1] & ~(1 << 7));	//LB2LO Disable
		
	}
	
    return TRUE;    
}

PUBLIC BOOL 
WavHal_AudioMessage(UINT32 Msg, UINT32 Param1, UINT32 Param2)
{
DEBUG1(DGB_LVL_IF,"+WavHal_AudioMessage\n");
    switch( Msg )
    {
        case WODM_GET_VOLUME:
            *((PUINT32) Param1) = v_nVolume;
            break;
        
        case WODM_SET_VOLUME:
            v_nVolume = (UINT32) Param1;
            WaveOutSetVolume(v_nVolume);
//			WaveOutSetPath(Param1);
            break;
//		case WODM_SET_PATH:
//            WaveOutSetPath((UINT32) Param1);
//			break;
        default:
            break;
    }

    return TRUE;
}
